/* Jungle Monkey
 * Copyright (C) 1999, 2000  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _B_CONN_H
#define _B_CONN_H

#include <stdio.h>
#include "jmgnet.h"
#include "b_types.h"


/* Maximum total size of MCAST packets in packet_queue. */
#define B_CONN_MAX_MCAST_SIZE	16384



/* ******************** */


BConn*   b_conn_new (gchar* hostname, gint port, guint protocol, gchar* object_name,
		     JMGTOS tos, BConnFuncs* funcs, gpointer user_data);
void	 b_conn_ref (BConn* conn);
void	 b_conn_unref (BConn* conn);
void     b_conn_delete (BConn* conn);
void	 b_conn_reset (BConn* conn);

void	 b_conn_print (FILE* file, BConn* conn);

void	 b_conn_func_connect   (BConn* conn);
void	 b_conn_func_ping      (BConn* conn);
void	 b_conn_func_writeable (BConn* conn);
void	 b_conn_func_fail      (BConn* conn);
void	 b_conn_func_close     (BConn* conn);


gboolean b_conn_has_packet (BConn* conn, guint8 type);
gboolean b_conn_has_packet_by_subtype (BConn* conn, guint8 type, guint8 subtype);

void 	 b_conn_send_packet (BConn* conn, BPacket* packet);
void	 b_conn_prepend_packet (BConn* conn, BPacket* packet);
void	 b_conn_append_packet (BConn* conn, BPacket* packet);

BPacket* b_conn_pop_packet (BConn* conn);
void	 b_conn_remove_packets (BConn* conn, guint8 type);
void	 b_conn_remove_packets_by_subtype (BConn* conn, guint8 type, guint8 subtype);


#define  b_conn_is_unconnected(CONN)	((CONN)->status == B_CONN_STATUS_UNCONNECTED)
#define  b_conn_is_closing(CONN)	((CONN)->status == B_CONN_STATUS_CLOSING)
#define  b_conn_is_connected(CONN)	((CONN)->status == B_CONN_STATUS_CONNECTED)
#define  b_conn_is_connecting(CONN)	((CONN)->status == B_CONN_STATUS_CONNECTING)
#define  b_conn_is_accepted(CONN)	((CONN)->status == B_CONN_STATUS_ACCEPTED)



/* Unref the CONN and return VAL if it's destroyed */
#define	 b_conn_unref_and_return_val_if_destroyed(CONN, VAL)	\
do {								\
  BConn* _myconn = (CONN);					\
  if (--_myconn->ref_count == 0)				\
    {								\
      b_conn_delete (_myconn);					\
      return (VAL);						\
    }								\
} while (0);



#endif /* _B_CONN_H */
