/* Jungle Monkey
 * Copyright (C) 1999, 2000  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _B_PACKET_H
#define _B_PACKET_H

#include "b_types.h"
#include "b_util.h"


#define B_PACKET_VERSION	0


typedef enum
{
  B_PACKET_PROTOCOL_BTP,
  B_PACKET_PROTOCOL_BYP,
  B_PACKET_PROTOCOL_BMP,
  B_PACKET_PROTOCOL_LAST

} BPacketProtocol;


typedef enum 
{ 
  B_PACKET_TYPE_OK,
  B_PACKET_TYPE_ERROR,
  B_PACKET_TYPE_CONNECT,
  B_PACKET_TYPE_BYE,

  B_PACKET_TYPE_UCAST,
  B_PACKET_TYPE_MCAST,		/* 5 */

  B_PACKET_TYPE_INFO_REQUEST,
  B_PACKET_TYPE_INFO_REPLY,

  B_PACKET_TYPE_LAST		/* 8 */

} BPacketType;


typedef enum 
{ 
  B_PACKET_INFO_SUBTYPE_PING,
  B_PACKET_INFO_SUBTYPE_LAST

} BPacketInfoSubtype;


typedef enum 
{ 
  B_PACKET_OK_SUBTYPE_NONE,
  B_PACKET_OK_SUBTYPE_CONNECT,
  B_PACKET_OK_SUBTYPE_LAST

} BPacketOkSubtype;


typedef enum 
{ 
  B_PACKET_ERROR_SUBTYPE_NONE,
  B_PACKET_ERROR_SUBTYPE_CONNECT,
  B_PACKET_ERROR_SUBTYPE_LAST

} BPacketErrorSubtype;



/* ******************** */

#define B_PACKET_LENGTH(P) (sizeof(BPacket) + g_ntohs((P)->length))


/* ******************** */

BPacket* b_packet_new (BConn* conn, guint8 type, guint16 length);
BPacket* b_packet_clone (BPacket* packet);
void	 b_packet_delete (BPacket* packet);

BPacket* b_packet_new_ok (BConn* conn, BPacketOkSubtype subtype);
BPacket* b_packet_new_ok_connect (BConn* conn);
BPacket* b_packet_new_error (BConn* conn, BPacketErrorSubtype subtype);
BPacket* b_packet_new_connect (BConn* conn, gchar* name);
BPacket* b_packet_new_bye (BConn* conn);
 		 
BPacket* b_packet_new_ucast (BConn* conn, guint8* buffer, guint16 length);
BPacket* b_packet_new_mcast (BConn* conn, guint8* buffer, guint16 length);
 		 
BPacket* b_packet_new_info_request_ping (BConn* conn);
BPacket* b_packet_new_info_reply_ping (BConn* conn, BPacket* ping);

gboolean b_packet_parse_ok (BPacket* packet, BUtilAddress** me_addressp, 
			    BUtilAddress** them_addressp, gchar** object_namep);


/* ******************** */

#endif /* _B_PACKET_H */
