/* Jungle Monkey
 * Copyright (C) 1999, 2000  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _B_TYPES_H
#define _B_TYPES_H

#include <glib.h>
#include <gnet/gnet.h>
#include "jmgnet.h"


/**
   
   BPacket.  All protocols use this structure.

*/
typedef struct _BPacket		/* 12 bytes long, 32 bit aligned */
{
  guint8 protocol;		/* Protocol 					*/
  guint8 version;		/* B_PACKET_VERSION			 	*/
  guint16 seq;			/* Sequence number (unused)			*/
  guint32 id;			/* Session ID	 	 			*/
  guint8 type;			/* Packet type					*/
  guint8 subtype;		/* Packet subtype				*/
  guint16 length;		/* Length of data				*/
  guint8 data[0];		/* Data						*/

} BPacket;



/* ******************** */

/**

   BConn represents a connection.

   Most BConn functions are in the file, but see also b_conn_connect
   and b_conn_accept.  (b_conn_ping and b_conn_io are internal).

 */



struct _BConn;
typedef struct _BConn BConn;


typedef enum _BConnStatus
{
  B_CONN_STATUS_UNCONNECTED,
  B_CONN_STATUS_CLOSING,
  B_CONN_STATUS_CONNECTED,
  B_CONN_STATUS_CONNECTING,
  B_CONN_STATUS_ACCEPTED

} BConnStatus;


typedef void	 (*BConnConnectFunc)   (BConn* conn, gpointer user_data);
typedef void	 (*BConnPingFunc)      (BConn* conn, gpointer user_data);
typedef void	 (*BConnWriteableFunc) (BConn* conn, gpointer user_data);
typedef void 	 (*BConnFailFunc)      (BConn* conn, gpointer user_data);
typedef void 	 (*BConnCloseFunc)     (BConn* conn, gpointer user_data);


typedef struct _BConnFuncs
{
  BConnConnectFunc	connect;
  BConnPingFunc		ping;
  BConnWriteableFunc	writeable;
  BConnCloseFunc	close;
  BConnFailFunc		fail;

} BConnFuncs;



struct _BConn
{
  JMGConn*		conn;
  gchar*		object_name;
  guint 		protocol;
  JMGTOS		tos;

  gint 			ref_count;
  BConnStatus		status;
  
  BConnFuncs*		funcs;
  gpointer		user_data;

  GList*		handlers;


  /* ******************** */

  gboolean		is_pinging;
  guint 		send_ping_timer;
  guint			conn_timeout_timer;
  guint			distance;	/* RTT in milliseconds				*/

  guint32 		id_in;		/* ID expected for incoming packets (set by us)	*/
  guint32 		id_out;		/* ID used for outgoing packets (set by them)	*/

  GList* 		packet_queue;
  guint			packet_queue_size;

  BPacket* 		current_packet;	/* Long packets being parsed		 	*/
  guint16 		total_read;	/* Bytes of current_packet read			*/

  guint 		buffer_length;	/* Offset for writing in the buffer		*/
  gchar 		buffer[4096];


};


#endif /*  _B_TYPES_H */
