/* Jungle Monkey
 * Copyright (C) 1999, 2000  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _BACKEND_H
#define _BACKEND_H

#include <glib.h>
#include <gnet/gnet.h>

#include "jmfile.h"
#include "mtp_server.h"

#include "backend_channel.h"
#include "backend_directory.h"
#include "backend_file.h"
#include "backend_search.h"
#include "backend_url.h"


#define JMP_UPDATE_TIME_MAX		200000
#define JMP_UPDATE_TIME_MIN		   100
/* TODO: This limits you to sending about 2000 files per channel.  If we
   aggregate we can do more.  */

#define GARBAGE_COLLECT_TIME		(JMP_UPDATE_TIME_MAX * 4)
#define EXPIRE_TIME			(GARBAGE_COLLECT_TIME * 2)

#define JM_DEFAULT_CHANNEL "jm://junglemonkey.net/Monkey Central"

extern char* jm_home;		/* Jungle Monkey home directory. */

/* set by backend_init */
extern gchar*	  backend_hostname;
extern gint	  backend_port;
extern GInetAddr* backend_interface;
extern MtpServer* backend_mtp_server;

/* may be set anywhere */
extern JMGURL*	  backend_rendezvous_url;
extern gboolean	  backend_rendezvous_locally;
extern gboolean   backend_auto_mirror_on_download;


/* ******************** */

/* Start the backend.  Return TRUE on success. */
gboolean backend_init (gchar* hostname, GInetAddr* interface, 
		       gint btp_port, gboolean force_btp_port,
		       gint mtp_port, gboolean force_mtp_port,
		       gboolean global_rendezvous);


/* Stop/shutdown the backend.  Returns TRUE if the backend is
   shutdown, FALSE if it needs more time.  */
gboolean backend_shutdown (void);

gboolean backend_is_initialized (void);


/* ******************** */

/* Add a file. (internal) */
JMFile* backend_add (gchar* name, gchar* url, JMFileType type, JMFile* parent);

/* Remove a file. */
void backend_remove (JMFile* file);

/* Unlink a file */
void backend_unlink (JMFile* file, JMFile* parent);

/* Activate a file.  What happens depends on the file and type. */
void backend_activate (JMFile* file);


/* ******************************************** */
/* Private backend stuff			*/

JMFile* backend_get (gchar* url);
void	backend_put (JMFile* jmfile);
void	backend_delete (JMFile* jmfile);


#endif /* _BACKEND_H */
