/* Jungle Monkey
 * Copyright (C) 1999, 2000  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "backend_search.h"
#include "backend_announce.h"

#include "gui.h"


JMFile* 
backend_search_create (gchar* name, gchar* url, JMFile* channel)
{
  JMFile* search;

  g_return_val_if_fail(name, NULL);
  g_return_val_if_fail(channel, NULL);
  g_return_val_if_fail(channel->type == JMFILE_CHANNEL, NULL);
  g_return_val_if_fail(url, NULL);

  /* Warn if we collide with another URL.  This could happen if we
     reset and recreate an old search. */
  if ((search = backend_get(url)) != NULL)
    {
      g_return_val_if_fail (search->is_local, NULL);

      g_warning ("Search with URL %s already exists, deleting.\n", url);
      jmfile_delete (search);
    }

  search = jmfile_new (name, JMFILE_SEARCH, url);
  search->is_local = TRUE;
  jmfile_link (search, channel);

  backend_put (search);
  backend_announce_add (search, channel);

  gui_add (search);

  return search;
}
