/* Jungle Monkey
 * Copyright (C) 1999, 2000  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "backend_url.h"
#include "backend_announce.h"

#include "gui.h"


JMFile* 
backend_url_create (gchar* name, gchar* url, JMFile* parent)
{
  JMFile* jmfile;

  g_return_val_if_fail (!parent || 
			(parent->type == JMFILE_CHANNEL || 
			 parent->type == JMFILE_DIR), NULL);
  g_return_val_if_fail (url, NULL);

  /* Warn if we collide with another URL.  This could happen if we
     reset and recreate an old url. */
  if ((jmfile = backend_get(url)) != NULL)
    {
      g_return_val_if_fail (jmfile->is_local, NULL);

      g_warning ("File with URL %s already exists, deleting.\n", url);
      jmfile_delete (jmfile);
    }

  /* Create a new file */
  jmfile = jmfile_new (name, JMFILE_URL, url);
  jmfile->is_local = TRUE;
  jmfile_link (jmfile, parent);

  backend_put (jmfile);

  /* Send an announcement, if I have a parent */
  if (parent)
    backend_announce_add (jmfile, parent);

  /* Add it to the GUI */
  gui_add (jmfile);

  return jmfile;
}


void
backend_url_activate (JMFile* jmfile)
{
  gui_activate (jmfile);
}
