/* Jungle Monkey
 * Copyright (C) 1999, 2000  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _BTP_H
#define _BTP_H


#include <glib.h>
#include <gnet/gnet.h>

#include "btp_tree.h"
#include "btp_node.h"


/*

  Upper-level interfaces for BTP.  See also some of the function in
  btp_tree.

  The Banana Tree Protocol is an end-to-end multicast tree creation
  protocol.  It tries to create a low-cost tree, not a low-delay tree.
  Right now BTP runs ontop of TCP, but it could just as well run ontop
  of UDP (with some additional tuning).

  It does:
    joining and leaving a parent
    information (degree limit, children)
    pinging and ponging

  It does not:
    transfer packets reliably
    eliminate duplicate packets
    request/retransmit lost packets

 */



/* **************************************** */

#define BTP_PORT	5656


/* FALSE = success */
gboolean btp_init (gchar* hostname, GInetAddr* listening_interface, gint port, gboolean force_port);
gboolean btp_is_initialized (void);
void 	 btp_shutdown (void);

BtpTree* btp_create (gchar* name, BtpTreeType tree_type);
BtpTree* btp_join (gchar* url, BtpTreeType tree_type, gboolean passive);

void	 btp_ref (BtpTree* tree);
gboolean btp_unref (BtpTree* tree);
gboolean btp_clear_and_unref (BtpTree* tree);

void 	 btp_send_mcast (BtpTree* tree, BtpNode* not_node, guint8* buffer, guint16 length);
void 	 btp_send_mcast_children (BtpTree* tree, guint8* buffer, guint16 length);
void 	 btp_send_ucast (BtpTree* tree, BtpNode* node, guint8* buffer, guint16 length);


/* **************************************** */
/* Functions for other modules		    */

BtpTree* btp_get (gchar* name);
void 	 btp_add (BtpTree* tree);
void 	 btp_remove (BtpTree* tree);



#endif /* _BTP_H */
