/* Jungle Monkey
 * Copyright (C) 1999, 2000  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "globals.h"

#include "b_types.h"
#include "b_conn.h"
#include "b_conn_connect.h"

#include "btp_node.h"
#include "btp_proto.h"
#include "btp_tree.h"



/* ************************************************************ */


BtpNode* 
btp_node_new (BtpTree* tree, gchar* hostname, gint port, BConn* conn)
{
  BtpNode* node;

  g_return_val_if_fail (tree, NULL);
  g_return_val_if_fail (hostname, NULL);
  g_return_val_if_fail (port, NULL);

  node = g_new0 (BtpNode, 1);
  node->tree = tree;
  node->hostname = g_strdup (hostname);
  node->port = port;

  if (conn)
    {
      node->conn = conn;
      conn->funcs = &btp_proto_funcs;
      conn->user_data = node;
    }
  else
    {
      JMGTOS tos;

      if (tree->type == BTP_TREE_TYPE_BULK)
	tos = JMGTOS_LOWCOST;
      else
	tos = JMGTOS_NONE;

      node->conn = b_conn_new (hostname, port, B_PACKET_PROTOCOL_BTP, tree->name, 
			       tos, &btp_proto_funcs, node);
    }

  btp_tree_add_node (node);

  return node;
}


void
btp_node_delete (BtpNode* node)
{
  g_return_if_fail (node);
  g_return_if_fail (!node->is_deleting);

  DEBUGP("btp_node_delete %p\n", node);

  /* Make sure we are completely removed from the tree */
  if (node->tree)
    {
      btp_tree_remove_child (node);
      btp_tree_remove_sibling (node);
      btp_tree_remove_node (node);

      if (node->tree->root == node)
	node->tree->root = NULL;

      if (node->tree->parent == node)
	node->tree->parent = NULL;

      if (node->tree->new_parent == node)
	node->tree->new_parent = NULL;
    }

  if (node->hostname)
    {
      g_free (node->hostname);
      node->hostname = NULL;
    }
  node->port = 0;

  if (node->conn)
    {
      if (b_conn_is_connected (node->conn))
	{
	  node->is_deleting = TRUE;
	  b_conn_close (node->conn);
	  return;
	}
      else
	{
	  b_conn_unref (node->conn);
	  node->conn = NULL;
	}
    }

  memset (node, 0, sizeof (*node));
  g_free (node);
}
