/* Jungle Monkey
 * Copyright (C) 1999, 2000  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <string.h>

#include "globals.h"

#include "b_util.h"

#include "btp_packet.h"
#include "btp_node.h"


/* **************************************** */



BPacket*
btp_packet_new_info_request_children (BtpNode* node)
{
  BPacket* packet;

  g_return_val_if_fail (node, NULL);
  g_return_val_if_fail (node->conn, NULL);

  packet = b_packet_new (node->conn, B_PACKET_TYPE_INFO_REQUEST, 0);
  packet->subtype = BTP_PACKET_INFO_SUBTYPE_CHILDREN;

  return packet;
}



BPacket*
btp_packet_new_info_reply_children (BtpNode* node, GList* children)
{
  GList* conns = NULL;
  gchar* children_str = NULL;
  guint length = 0;
  BPacket* packet;

  g_return_val_if_fail (node, NULL);
  g_return_val_if_fail (node->conn, NULL);

  while (children)
    {
      conns = g_list_prepend (conns, ((BtpNode*) children->data)->conn);
      children = children->next;
    }
  conns = g_list_reverse (conns);
  children_str = b_util_write_addresses (conns, &length);
  g_list_free (conns);

  packet = b_packet_new (node->conn, B_PACKET_TYPE_INFO_REPLY, length);
  packet->subtype = BTP_PACKET_INFO_SUBTYPE_CHILDREN;
  memcpy (packet->data, children_str, length);

  g_free (children_str);

  return packet;
}




BPacket*
btp_packet_new_join (BtpNode* node)
{
  BPacket* packet;

  g_return_val_if_fail (node, NULL);
  g_return_val_if_fail (node->conn, NULL);

  packet = b_packet_new (node->conn, BTP_PACKET_TYPE_JOIN, 0);

  return packet;
}



BPacket*
btp_packet_new_switch (BtpNode* node, BtpNode* parent)
{
  BPacket* packet;
  gchar* parent_address;
  guint length;

  g_return_val_if_fail (node, NULL);
  g_return_val_if_fail (node->conn, NULL);
  g_return_val_if_fail (parent, NULL);
  g_return_val_if_fail (parent->conn, NULL);


  /* Get the address of the parent */
  parent_address = b_util_write_address (parent->conn, &length);
  g_return_val_if_fail (parent_address != NULL, NULL);

  /* Create the packet */
  packet = b_packet_new (node->conn, BTP_PACKET_TYPE_SWITCH, length);
  memcpy (packet->data, parent_address, length);
  g_free (parent_address);

  return packet;
}



BPacket*
btp_packet_new_leave (BtpNode* node)
{
  BPacket* packet;

  g_return_val_if_fail (node, NULL);
  g_return_val_if_fail (node->conn, NULL);

  packet = b_packet_new (node->conn, BTP_PACKET_TYPE_LEAVE, 0);

  return packet;
}

