/* Jungle Monkey
 * Copyright (C) 1999, 2000  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _BTSMP_H
#define _BTSMP_H

#include <glib.h>


#define BTSMP_MAX_PACKET_SIZE	1200	/* This is arbitrary... */

/**

   Banana Tree Simple Multicast Protocol (BTSMP).  A simple protocol
   for unreliable multicast transfer built ontop of BTP.

   BTP hints used:
     BTPHINT_NODE_JOIN	     If we receive a join, we send all packets
			       in the cache to the new node via unicast.

 */

typedef struct _BTSmp BTSmp;

typedef gboolean (*BTSmpPacketFunc)(BTSmp* btsmp, guint8* buffer, guint length, gpointer user_data);
typedef void (*BTSmpErrorFunc) (BTSmp* btsmp, gpointer user_data);

/* BTSmpPacketFunc returns TRUE if the packet is OK, FALSE if it is
   invalid.  If it is invalid, it will not be cached or forwarded. */


struct _BTSmp
{
  gchar* url;
  struct _BtpTree* tree;

  /* Upper level callbacks (see above)   */
  BTSmpPacketFunc packet_func;
  BTSmpErrorFunc  error_func;

  gpointer packet_user_data;
  gpointer error_user_data;

  GList* cache;
  guint cache_size;
  guint max_cache_size;
  guint cache_timeout;
};


BTSmp* btsmp_create (gchar* groupname);

BTSmp* btsmp_join (gchar* url);

void   btsmp_delete (BTSmp* btsmp);

void   btsmp_send_packet (BTSmp* btsmp, guint8* buffer, guint length);

/* Set max cache size and cache entry timeout (in ms) */
void   btsmp_cache_set_policy (BTSmp* btsmp, guint max_cache_size, guint cache_timeout);

void   btsmp_cache_clear (BTSmp* btsmp);

gboolean  btsmp_is_up (BTSmp* btsmp);


#endif /* _BTSMP_H */
