/* Jungle Monkey
 * Copyright (C) 1999, 2000  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "ggui.h"
#include "ggui_channels.h"
#include "ggui_chat.h"
#include "ggui_config.h"
#include "ggui_download.h"
#include "ggui_files.h"
#include "ggui_menu.h"
#include "ggui_preferences.h"
#include "ggui_properties.h"
#include "ggui_search.h"

#include "jmgnet.h"
#include "globals.h"
#include "util.h"
#include "icons.h"
#include "crp.h"

void ggui_set_tooltip(GtkWidget* widget, const gchar* tip);

gboolean on_jmw_destroy (GtkWidget* widget, gpointer user_data);
static gboolean final_destroy_cb(gpointer user_data);

static void crp_func (const gchar* hostname, gint port, CRPStatus status, 
		      GInetAddr* addr, gpointer user_data);



/* ******************** */
/* Pixmaps		*/

static GdkPixmap* page_pixmap 		= NULL;
static GdkBitmap* page_mask 		= NULL;
static GdkPixmap* gray_page_pixmap 	= NULL;
static GdkBitmap* gray_page_mask 	= NULL;
static GdkPixmap* down_page_pixmap 	= NULL;
static GdkBitmap* down_page_mask 	= NULL;

static GdkPixmap* folder_pixmap 	= NULL;
static GdkBitmap* folder_mask 		= NULL;

static GdkPixmap* channel_pixmap 	= NULL;
static GdkBitmap* channel_mask 		= NULL;
static GdkPixmap* gray_channel_pixmap 	= NULL;
static GdkBitmap* gray_channel_mask 	= NULL;
static GdkPixmap* down_channel_pixmap 	= NULL;
static GdkBitmap* down_channel_mask 	= NULL;

static GdkPixmap* chat_pixmap	 	= NULL;
static GdkBitmap* chat_mask 		= NULL;
static GdkPixmap* gray_chat_pixmap 	= NULL;
static GdkBitmap* gray_chat_mask 	= NULL;
static GdkPixmap* down_chat_pixmap 	= NULL;
static GdkBitmap* down_chat_mask 	= NULL;

static GdkPixmap* url_pixmap 		= NULL;
static GdkBitmap* url_mask 		= NULL;

GdkPixmap* back_pixmap 		= NULL;
GdkBitmap* back_mask 		= NULL;


/* ******************** */
/* GUI			*/

GladeXML* 	jmw_xml = NULL;
GtkWidget* 	jmw;


/* **************************************** */


static gboolean shutting_down = FALSE;
GtkWidget*	jmw_notebook;

static gchar*	crp_server = "crp://junglemonkey.net";



int
main(int argc, char* argv[])
{
  gint is_channels_only = FALSE;
  gchar* root_name = NULL;
  gchar* url = NULL;
  JMFile* root_channel = NULL;

  gchar* hostname = NULL;
  gchar* hostname_name = NULL;

  GInetAddr* interface = NULL;
  gchar* interface_name = NULL;
  gboolean use_dns = FALSE;

  gint btp_port = 0;
  gboolean force_btp_port = FALSE;

  gint mtp_port = 0;
  gboolean force_mtp_port = FALSE;

  gchar* rendezvous_url_str = NULL;
  gboolean bad_network_config = FALSE;

  GdkColor transparent;

  struct poptOption jm_options[] = 
  {
#ifndef JM_ENABLE_GNOME
    {"help", 'h', POPT_ARG_NONE, NULL, 1, 
     "Show this help message", NULL},
    {"usage", '\0', POPT_ARG_NONE, NULL, 2, 
     "Display brief usage message", NULL},
    {"version", 'V', POPT_ARG_NONE, NULL, 3,
     "Output verson information and exit", NULL},
#endif

    {"debug-level", 'd', POPT_ARG_INT, &debug_level, 0, 
     "Set the debug level", "LEVEL"},

    {"home", '\0', POPT_ARG_STRING, &jm_home, 0, 
     "Set the JM home directory (default is ~/jm)", "HOME"},

    {"hostname", '\0', POPT_ARG_STRING, &hostname_name, 0,
     "Hostname or address of this host", "HOSTNAME"},
    {"use-dns", '\0', POPT_ARG_NONE, &use_dns, 0,
     "Use DNS to figure out my actual hostname", NULL},

    {"interface", '\0', POPT_ARG_STRING, &interface_name, 0,
     "Interface address for listening socket", "ADDR"},

    {"crp-server", '\0', POPT_ARG_STRING, &crp_server, 0,
     "URL of CRP server (default is crp://junglemonkey.net)", "URL"},

    {"btp-port", '\0', POPT_ARG_STRING, &btp_port, 0,
     "Port for BTP listening socket", "PORT"},
    {"force-btp-port", '\0', POPT_ARG_NONE, &force_btp_port, 0,
     "Fail if can't set MTP port", NULL},

    {"mtp-port", '\0', POPT_ARG_STRING, &mtp_port, 0,
     "Port for MTP listening socket", "PORT"},
    {"force-mtp-port", '\0', POPT_ARG_NONE, &force_mtp_port, 0,
     "Fail if can't set MTP port", NULL},

    {"rendezvous", '\0', POPT_ARG_STRING, &rendezvous_url_str, 0,
     "URL of rendezvous server", "URL"},

    {"create-channel", 'r', POPT_ARG_STRING, &root_name, 0, 
     "Create a channel with the given name", "NAME"},
    {"channels-only", 'c', POPT_ARG_NONE, &is_channels_only, 0, 
     "Allow only channel advertisements in the created channel", NULL},

    {NULL, '\0', 0, NULL, 0, NULL, NULL}
  };

  poptContext ctx;


  /* ******************** */
  /* Library init	  */

#ifdef ENABLE_NLS
  {
    gtk_set_locale();
    bindtextdomain (PACKAGE, LOCALEDIR);
    textdomain (PACKAGE);
  }
#endif


#ifdef JM_ENABLE_GNOME
  {

    gnome_init_with_popt_table ("Jungle Monkey", VERSION, 
				argc, argv,
				jm_options, 0, &ctx);
    glade_gnome_init();
  }
#else
  {
    int rc;

    gtk_init(&argc, &argv);
    glade_init();

    /* ******************** */
    /* Parse command line	  */
    ctx = poptGetContext(NULL, argc, argv, jm_options, 0);
    poptSetOtherOptionHelp(ctx, _("[OPTION]... [URL]"));

    while ((rc = poptGetNextOpt(ctx)) > 0)
      {
	switch (rc)
	  {
	  case 1:	  goto help;	  break;
	  case 2:	  goto usage;	  break;
	  case 3:   	  goto version;	  break;
	  }
      }
  }
#endif

  /* Get the URL */
  url = poptGetArg(ctx);
  poptFreeContext(ctx);

  /* Load the configuration file */
  ggui_config_init ();


  /* ******************** */
  /* Initialize GUI	  */

  /* Load the XML */
#ifdef JM_ENABLE_GNOME
  jmw_xml = ggui_get_jm_glade_xml("jmw", "jm-gnome.glade");
#else
  jmw_xml = ggui_get_jm_glade_xml("jmw", "jm.glade");
#endif
  g_assert (jmw_xml != NULL);
  glade_xml_signal_autoconnect(jmw_xml);	


  jmw = glade_xml_get_widget(jmw_xml, "jmw");
  jmw_notebook = glade_xml_get_widget(jmw_xml, "jmw_notebook");

  g_assert (jmw != NULL);
  g_assert (jmw_notebook != NULL);


  /* Load the pixmaps */
  channel_pixmap = gdk_pixmap_create_from_xpm_d (jmw->window, &channel_mask,
						 &transparent, channel_xpm);
  gray_channel_pixmap = 
    gdk_pixmap_create_from_xpm_d (jmw->window, &gray_channel_mask,
				  &transparent, gray_channel_xpm);
  down_channel_pixmap = 
    gdk_pixmap_create_from_xpm_d (jmw->window, &down_channel_mask,
				  &transparent, down_channel_xpm);

  page_pixmap = 
    gdk_pixmap_create_from_xpm_d (jmw->window, &page_mask,
				  &transparent, page_xpm);
  gray_page_pixmap = 
    gdk_pixmap_create_from_xpm_d (jmw->window, &gray_page_mask,
				  &transparent, gray_page_xpm);
  down_page_pixmap = 
    gdk_pixmap_create_from_xpm_d (jmw->window, &down_page_mask,
				  &transparent, down_page_xpm);

  chat_pixmap = 
    gdk_pixmap_create_from_xpm_d (jmw->window, &chat_mask,
				  &transparent, chat_xpm);
  gray_chat_pixmap = 
    gdk_pixmap_create_from_xpm_d (jmw->window, &gray_chat_mask,
				  &transparent, gray_chat_xpm);
  down_chat_pixmap = 
    gdk_pixmap_create_from_xpm_d (jmw->window, &down_chat_mask,
				  &transparent, down_chat_xpm);

  url_pixmap = 
    gdk_pixmap_create_from_xpm_d (jmw->window, &url_mask,
				  &transparent, url_xpm);

  folder_pixmap = 
    gdk_pixmap_create_from_xpm_d (jmw->window, &folder_mask,
				  &transparent, folder_closed_xpm);

  back_pixmap = 
    gdk_pixmap_create_from_xpm_d (jmw->window, &back_mask,
				  &transparent, back_xpm);


  g_assert (channel_pixmap && channel_mask);
  g_assert (gray_channel_pixmap && gray_channel_mask);
  g_assert (down_channel_pixmap && down_channel_mask);

  g_assert (page_pixmap && page_mask);
  g_assert (gray_page_pixmap && gray_page_mask);
  g_assert (down_page_pixmap && down_page_mask);

  g_assert (chat_pixmap && chat_mask);
  g_assert (gray_chat_pixmap && gray_chat_mask);
  g_assert (down_chat_pixmap && down_chat_mask);

  g_assert (url_pixmap && url_mask);
  g_assert (folder_pixmap && folder_mask);
  g_assert (back_pixmap && back_mask);


  /* Start up other GUI modules */
  ggui_channels_init ();
  ggui_chat_init ();
  ggui_download_init ();
  ggui_files_init ();
  ggui_menu_init ();
  ggui_preferences_init ();
  ggui_properties_init ();
  ggui_search_init ();


  /* **************************************** */
  /* Warn if I'm root 			      */

  if (getuid() == 0)
    {
      ggui_show_message ("warning", 
			 _("DO NOT RUN JUNGLE MONKEY AS ROOT! \n"
			   "If you do, you're asking for trouble."));
    }


  /* **************************************** */
  /* Initialize the backend 		      */


  /* Set some options based on what's in the configuration file */
  if (!interface_name &&
      !ggui_config_get_bool("/Jungle Monkey/Network/Auto detect interface", TRUE))
    {
      interface_name = 
	ggui_config_get_string("/Jungle Monkey/Network/Interface", NULL);
    }

  if (!hostname_name &&
      !ggui_config_get_bool("/Jungle Monkey/Network/Auto detect hostname", TRUE))
    {
      hostname_name = 
	ggui_config_get_string("/Jungle Monkey/Network/Hostname", NULL);
    }

  if (ggui_config_get_bool("/Jungle Monkey/Network/Auto detect hostname", TRUE))
    {
      if (!use_dns)
	use_dns = 
	  ggui_config_get_bool("/Jungle Monkey/Network/Use DNS", FALSE);
    }

  if (!btp_port && 
      !ggui_config_get_bool("/Jungle Monkey/Channels"
			    "/Use Default BTP Port", TRUE))
    {
      btp_port = 
	ggui_config_get_int("/Jungle Monkey/Channels/BTP Port", 0);
    }
  if (!force_btp_port)
    {
      force_btp_port = 
	!ggui_config_get_bool("/Jungle Monkey/Channels"
			      "/Use any BTP Port if unavailable", TRUE);
    }

  if (!mtp_port && 
      !ggui_config_get_bool("/Jungle Monkey/Files"
			    "/Use Default MTP Port", TRUE))
    {
      mtp_port = 
	ggui_config_get_int("/Jungle Monkey/Files/MTP Port", 0);
    }
  if (!force_mtp_port)
    {
      force_mtp_port = 
	!ggui_config_get_bool("/Jungle Monkey/Files"
			      "/Use any MTP Port if unavailable", TRUE);
    }

  backend_rendezvous_locally = 
    ggui_config_get_bool("/Jungle Monkey/Files/Rendezvous locally", TRUE);
  if (!rendezvous_url_str && !backend_rendezvous_locally)
    {
      rendezvous_url_str = 
	ggui_config_get_string("/Jungle Monkey/Files/Rendezvous URL", NULL);
    }
    
  backend_auto_mirror_on_download = 
    ggui_config_get_bool ("/Jungle Monkey/Files/Automirror", TRUE);


  /* If an interface was specified, make sure it resoves to an address */
  if (interface_name)
    {
      /* Convert interface_name to InetAddr. */
      interface = gnet_inetaddr_new (interface_name, 0);
      if (!interface)
	{
	  ggui_disable_gui();

	  ggui_show_message("error", 
			    _("Bad interface address: %s. \n"
			      "The address must be of an interface on your \n"
			      "machine, preferably in dotted decimal notation.\n"
			      "Use /sbin/ifconfig to list the interfaces on \n"
			      "your machine. You can set the interface \n"
			      "manually under Settings/Preferences/Network. \n"
			      "JM will not work until this is fixed."), 
			    interface_name);

	  bad_network_config = TRUE;
	  goto skip_network;
	}
    }

  /* Otherwise, auto-detect the interface */
  else
    {
      interface = jm_gnet_inetaddr_autodetect_interface ();
      if (!interface)
	{
	  ggui_disable_gui();

	  ggui_show_message("error", 
			    _("Could not auto-detect an internet interface. \n"
			      "You can set the interface manually under \n"
			      "Settings/Preferences/Network or by using \n"
			      "the --interface command line option. Use \n"
			      "/sbin/ifconfig to list the interfaces on your \n"
			      "machine. JM will not work until this is \n"
			      "fixed."));

	  bad_network_config = TRUE;
	  goto skip_network;
	}
    }


  /* If a hostname was specified, use it */
  if (hostname_name)
    {
      hostname = g_strdup (hostname_name);
    }

  /* Autodetect the hostname */
  else
    {
      if (use_dns)
	hostname = gnet_inetaddr_get_name (interface);

      if (!hostname ||
	  !jm_gnet_inetaddr_is_internet_domainname (hostname))
	hostname = gnet_inetaddr_get_canonical_name (interface);

      g_assert (hostname);

      if (!jm_gnet_inetaddr_is_internet_domainname (hostname))
	{
	  ggui_disable_gui();

	  ggui_show_message("error", 
			    _("Could not auto-detect the hostname. \n"
			      "We detected %s, but it does not \n"
			      "look like a valid internet hostname. You \n"
			      "can set the address manually under \n"
			      "Settings/Preferences/Network or by using the \n"
			      "--hostname command line option. JM will not \n"
			      "work until this is fixed."), hostname);

	  g_free (hostname);
	  hostname = NULL;

	  bad_network_config = TRUE;
	  goto skip_network;
	}
    }

 skip_network:

  if (root_name == NULL && url == NULL)
    {
      url = JM_DEFAULT_CHANNEL;
    }

  /* Check arguments */
  if (root_name != NULL && url != NULL)
    {
      ggui_show_message ("error", _("You cannot create a private channel and \n"
				    "join a private channel at the same time"));
      root_name = NULL;
    }

  if (rendezvous_url_str)
    {
      backend_rendezvous_url = jm_url_new (rendezvous_url_str);
      if (!backend_rendezvous_url)
	ggui_show_message ("warning", _("Bad rendezvous URL: %s"), rendezvous_url_str);
      jm_url_set_protocol (backend_rendezvous_url, "mtp");
    }


  /* Set the Jungle Monkey home directory.  This is set in this order:
     
     1) command line
     2) Gnome config (if Gnome support)
     3) HOME + ~/


     Try to get it from the JMHOME environment variable.  If that
     isn't set, use HOME/jm.

  */
  if (jm_home == NULL)
    {
      jm_home = ggui_config_get_string("/Jungle Monkey/General/home", NULL);
      if (jm_home == NULL)
	{
	  jm_home = g_get_home_dir(); 	/* don't delete */
	  g_assert (jm_home != NULL);

	  jm_home = g_strconcat(jm_home, G_DIR_SEPARATOR_S, "jm", NULL);
	}
    }

  /* Strip off the trailing / if necesssary */
  if (jm_home && *jm_home && jm_home[strlen(jm_home)-1] == G_DIR_SEPARATOR)
    jm_home[strlen(jm_home)-1] = '\0';

  /* Create the JM home directory if it doesn't exist. */
  if (!mkdirr(jm_home, 0700))
    {
      ggui_show_message ("warning", _("Cannot create Jungle Monkey home directory \"%s\""), 
			 jm_home);
    }


  /* ******************** */
  /* Initialize backend	  */

  if (backend_init (hostname, interface, btp_port, force_btp_port, 
		    mtp_port, force_mtp_port, FALSE))
    {
      ggui_disable_gui();

      /* Don't show another error if we already showed one. */
      if (!bad_network_config)
	ggui_show_message ("error", 
			   _("Could not start backend. \n"
			     "The network configuration may be bad. \n"
			     "Please check it under \n"
			     "Settings/Preferences/Network"));
    }
  else
    {
      /* Check the configuration */
      ggui_check_network_config();

      /* Set rendezvous url to local address if not set before */
      if (!backend_rendezvous_url)
	{
	  backend_rendezvous_url = 
	    jm_url_new_fields ("mtp", hostname, backend_mtp_server->server->port, NULL);
	}
    }


  /* Create 'first' channel.  This is either the default channel or
     the channel specified on the command line.  */
  if (root_name)
    {
      root_channel = backend_channel_create (root_name, NULL, is_channels_only);
      if (root_channel)
	ggui_channels_show (root_channel);
      else
	ggui_show_message ("warning", _("Couldn't create root channel %s"), root_name);
    }
  else
    {
      root_channel = backend_channel_add (url, NULL);
      if (root_channel)
	{
	  ggui_channels_show (root_channel);

	  /* Bookmark if it's the default channel */
	  if (!strcmp(url, JM_DEFAULT_CHANNEL))
	    ggui_channels_set_bookmark (root_channel, TRUE);
	}
      else
	ggui_show_message ("warning", _("Couldn't join channel %s"), url);
    }

  /* Load bookmarked channels and chats */
  ggui_channels_load_bookmarks (root_channel);
  ggui_chat_load_bookmarks ();

  gtk_main();

  return 0;

#ifndef JM_ENABLE_GNOME
 usage:
  poptPrintUsage(ctx, stdout, 0);
  exit(EXIT_SUCCESS);

 help:
  g_print (_("`jm' - Jungle Monkey, a distributed file sharing program\n"));
  poptPrintHelp(ctx, stdout, 0);
  g_print (_("\n"
	     "How to:\n"
	     "  Join Monkey_Central, the default root channel (if you're new, do this)\n"
	     "    jm\n"
	     "  Join some other channel\n"
	     "    jm jm://my-url\n"
	     "  Create my own root channel\n"
	     "    jm -r my_root_channel\n"
	     "\n"
	     "Report bugs to <jm-dev@umich.edu>\n"));
  exit(EXIT_SUCCESS);

 version:
  g_print (_("jm %s\n"
	     "Written by David A. Helder\n"
	     "\n"
	     "Copyright (C) 2000 The Regents of the University of Michigan\n"
	     "This is free software; see the source for copying conditions.  There is NO\n"
	     "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"),
	   VERSION);
  exit(EXIT_SUCCESS);
#endif
}


void
ggui_set_tooltip(GtkWidget* widget, const gchar* tip)
{
  GtkTooltips* tooltip;

  tooltip = gtk_tooltips_new();

  gtk_tooltips_set_tip(tooltip, widget, tip, NULL);
}


GuiData*
ggui_get_gui_data(JMFile* jmfile)
{
  g_return_val_if_fail (jmfile, NULL);

  if (!jmfile->gui_data)
    jmfile->gui_data = g_new0(GuiData, 1);

  return jmfile->gui_data;
}



/*

  Load <glade_filename>, which should have widget <name> in it.  If we
  can't load the file, we fail.

 */
GladeXML* 
ggui_get_jm_glade_xml(gchar* name, gchar* glade_filename)
{
  GladeXML* gxml = NULL;
  gchar* jm_glade;

  /* First check local directory */
  jm_glade = g_strconcat(".", G_DIR_SEPARATOR_S, 
			 "glade", G_DIR_SEPARATOR_S, 
			 glade_filename, NULL);
  if (file_exists(jm_glade))
    {
      gxml = glade_xml_new(jm_glade, name);
    }
  g_free  (jm_glade);

  if (gxml == NULL)
    {
      jm_glade = g_strconcat(JM_DATADIR, G_DIR_SEPARATOR_S, 
			     "glade", G_DIR_SEPARATOR_S, 
			     glade_filename, NULL);

      if (file_exists(jm_glade))
	gxml = glade_xml_new(jm_glade, name);

      g_free (jm_glade);
    }

  if (gxml == NULL)
    {
      ggui_show_message ("error", _("Could not load glade file %s. "
				    "We looked in the directories ./glade and %s/glade. "
				    "Maybe JM is not correctly installed or libglade is "
				    "out-of-date."), glade_filename, JM_DATADIR);
    }

  return gxml;
}


#ifndef JM_ENABLE_GNOME

static void
on_messagebox_ok (gpointer button, gpointer dialog)
{
  gtk_widget_destroy (GTK_WIDGET(dialog));
}

#endif



void 
ggui_show_message(const gchar* type, const gchar* format, ...)
{
  va_list args;
  gchar* message;

  g_return_if_fail (type != NULL);
  g_return_if_fail (format != NULL);

  va_start (args, format);
  message = g_strdup_vprintf(format, args);
  va_end (args);

  g_return_if_fail (message != NULL);

#ifdef JM_ENABLE_GNOME
  {
    GtkWidget* message_box;

    message_box = gnome_message_box_new (message, type, GNOME_STOCK_BUTTON_OK, NULL);
    if (jmw)
      gnome_dialog_set_parent (GNOME_DIALOG(message_box), GTK_WINDOW(jmw));
    gtk_widget_show(message_box);
  }
#else
  {
    GtkWidget* dialog;
    GtkWidget* label;
    GtkWidget* ok;

    dialog = gtk_dialog_new();
    GTK_WINDOW(dialog)->position = GTK_WIN_POS_MOUSE;

    label = gtk_label_new (message);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, FALSE, FALSE, 4);
    gtk_widget_show(label);

    ok = gtk_button_new_with_label (_("OK"));
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), ok, FALSE, FALSE, 4);
    gtk_signal_connect (GTK_OBJECT (ok), "clicked",
			GTK_SIGNAL_FUNC(on_messagebox_ok), (gpointer) dialog);  
    gtk_widget_show (ok);


    if (!strcmp(type, "info"))
      {
	gtk_window_set_title(GTK_WINDOW(dialog), _("INFORMATION"));
      }
    else if (!strcmp(type, "warning"))
      {
	gtk_window_set_title(GTK_WINDOW(dialog), _("WARNING"));
      }
    else if (!strcmp(type, "error"))
      {
	gtk_window_set_title(GTK_WINDOW(dialog), _("ERROR"));
      }
    else
      {
	gtk_window_set_title(GTK_WINDOW(dialog), _("Message"));
      }

    gtk_widget_show(dialog);

  }
#endif

  g_free (message);
}



void
ggui_enable_gui(void)
{
  g_return_if_fail (jmw_notebook != NULL);
  
  ggui_menu_enable_menubar();
  
  gtk_widget_set_sensitive (jmw_notebook, TRUE);
}


void
ggui_disable_gui(void)
{
  g_return_if_fail (jmw_notebook != NULL);
  
  ggui_menu_disable_menubar();
  
  gtk_widget_set_sensitive (jmw_notebook, FALSE);
}


/* ************************************************************ */
/* Shared utility functions					*/

void
ggui_get_pixmaps(JMFile* jmfile, GdkPixmap** pixmapp, GdkBitmap** maskp)
{
  GdkPixmap* pixmap 	= NULL;
  GdkBitmap* mask	= NULL;

  g_return_if_fail (jmfile != NULL);

  switch (jmfile->type)
    {
    case JMFILE_CHANNEL:
      {
	if (jmfile_on_channel(jmfile))
	  {
	    pixmap = channel_pixmap;
	    mask   = channel_mask;
	  }
	else if (jmfile_is_down(jmfile))
	  {
	    pixmap = down_channel_pixmap;
	    mask   = down_channel_mask;
	  }
	else
	  {
	    pixmap = gray_channel_pixmap;
	    mask   = gray_channel_mask;
	  }
	break;
      }

    case JMFILE_DIR:
      {
	pixmap = folder_pixmap;
	mask   = folder_mask;
	break;
      }

    case JMFILE_FILE:
      {
	if (jmfile_have_file(jmfile))
	  {
	    pixmap = page_pixmap;
	    mask   = page_mask;
	  }
	else if (jmfile_is_down(jmfile))
	  {
	    pixmap = down_page_pixmap;
	    mask   = down_page_mask;
	  }
	else
	  {
	    pixmap = gray_page_pixmap;
	    mask   = gray_page_mask;
	  }

	break;
      }

    case JMFILE_URL:
      {
	if (jmfile_is_url_proto(jmfile, "jmchat"))
	  {
	    if (ggui_chat_on (jmfile))
	      {
		pixmap = chat_pixmap;
		mask   = chat_mask;
	      }
	    else if (ggui_chat_is_down (jmfile))
	      {
		pixmap = down_chat_pixmap;
		mask   = down_chat_mask;
	      }
	    else
	      {
		pixmap = gray_chat_pixmap;
		mask   = gray_chat_mask;
	      }

	    break;
	  }

	pixmap = url_pixmap;
	mask   = url_mask;

	break;
      }

    default:
      g_return_if_fail(FALSE);
    }

  if (pixmapp)	*pixmapp = pixmap;
  if (maskp)    *maskp   = mask;
}
		 



void
ggui_clist_update_row2(GtkCList* clist, gint row,
		       gchar* text1, gchar* text2,
		       GdkPixmap* pixmap, GdkBitmap* mask)
{
  gchar*     old_text;			/* don't delete */
  GdkPixmap* old_pixmap;
  GdkBitmap* old_mask;

  g_return_if_fail (clist != NULL);
  g_return_if_fail (row >= 0);

  gtk_clist_get_pixtext (clist, row, 0, &old_text, NULL,
			 &old_pixmap, &old_mask);

  if ((!old_text && text1) ||
      (old_text && !text1) || 
      (old_text && text1 && strcmp(old_text, text1)) ||
      (old_pixmap != pixmap)  ||
      (old_mask   != mask))
    {
      gtk_clist_set_pixtext (clist, row, 0, text1, 5,
			     pixmap, mask);
    }


  gtk_clist_get_text (clist, row, 1, &old_text);

  if ((!old_text && text2) || 
      (old_text && !text2) || 
      (old_text && text2 && strcmp(old_text, text2)))
    {
      gtk_clist_set_text (clist, row, 1, text2);
    }
}






/* ************************************************************ */
/* GUI signal code						*/

gboolean
on_jmw_destroy(GtkWidget* widget, gpointer user_data)
{
  if (!shutting_down)
    {
      shutting_down = TRUE;

      /* Shutdown GUI's */
      ggui_channels_shutdown ();
      ggui_chat_shutdown ();
      ggui_preferences_shutdown ();
      ggui_properties_shutdown ();

      /* Save jmhome */
      ggui_config_set_string ("/Jungle Monkey/General/home", jm_home);
      ggui_config_shutdown ();

      /* Close the window */
      g_assert (jmw != NULL);

      gtk_widget_destroy(jmw);

      if (!backend_is_initialized() || backend_shutdown())
	{
	  gtk_main_quit();
	  return TRUE;
	}
      else
	{
	  g_timeout_add(5000, final_destroy_cb, NULL);

	  /* TODO: Find a better way to do this.  If we can leave all our
	     channels immediately, then do so.  */
	}
    }

  return TRUE;	/* we handled the signal */
}


gboolean
final_destroy_cb(gpointer user_data)
{
  gtk_main_quit();
  return FALSE;
}


/* ******************** */

void
ggui_check_network_config (void)
{
  JMGURL* crp_url;

  /* Check the BTP hostname and port.  We won't bother checking MTP
     for now. */

  g_return_if_fail (backend_hostname);
  g_return_if_fail (backend_interface);
  g_return_if_fail (crp_server);

  crp_url = jm_url_new (crp_server);
  g_return_if_fail (crp_url);

  crp_check (crp_url, backend_hostname, backend_port, crp_func, NULL);

  jm_url_delete (crp_url);
}



static void 
crp_func (const gchar* hostname, gint port, CRPStatus status, 
	  /* callee owns */ GInetAddr* addr, gpointer user_data)
{
  gchar* addr_str = NULL;

  g_return_if_fail (backend_mtp_server);

  if (addr)
    addr_str = gnet_inetaddr_get_canonical_name (addr);

  switch (status)
    {
    case CRP_STATUS_BAD_PAIR:
      ggui_show_message ("warning", 
			 _("JM is configured to use %s as your address \n"
			   "and %d as the port for incoming connections. \n"
			   "However, the CRP server thinks your hostname is \n"
			   "%s and cannot connect to port %d. \n"
			   "If you do not fix this, other JM users will not \n"
			   "be able to connect to you. If you are behind a \n"
			   "firewall or NAT, you need to configure it to allow \n"
			   "connections to pass through to this port and \n"
			   "port %d. Please reconfigure JM or contact your \n"
			   "network administrator."),
			 hostname, port, addr_str, port, 
			 backend_mtp_server->server->port);
      break;

    case CRP_STATUS_BAD_NAME:
      ggui_show_message ("warning", 
			 _("JM is configured to use %s as your address. \n"
			   "However, the CRP server thinks your address is \n"
			   "%s. If you do not fix this, other JM users \n"
			   "will not be able to connect to you. Please \n"
			   "reconfigure and restart JM."),
			 hostname, addr_str);
      break;

    case CRP_STATUS_BAD_PORT:
      ggui_show_message ("warning", 
			 _("JM is configured to use %d as the port for \n"
			   "incoming connections. However, the CRP server \n"
			   "cannot connect to this port. If you do not fix \n"
			   "this, other JM users will not be able to connect \n"
			   "to you. If you are behind a firewall or NAT, you \n"
			   "need to configure it to allow connections to pass \n"
			   "through to this port and port %d. Contact your \n"
			   "network administrator if you do not know how to \n"
			   "do this."),
			 port, backend_mtp_server->server->port);
      break;


    case CRP_STATUS_GOOD:
    case CRP_STATUS_ERROR:
      ;
    }

  gnet_inetaddr_delete (addr);
  g_free (addr_str);
}





/* ********** */

void
on_open_channel(gpointer callback_data, guint callback_action, GtkWidget* widget)
{
  ggui_menu_open_channel();
}

void
on_create_root_channel(gpointer callback_data, guint callback_action, GtkWidget* widget)
{
  ggui_menu_create_root_channel();
}


# ifdef JM_ENABLE_GNOME

void
on_mtp_server_information (gpointer callback_data, guint callback_action, GtkWidget* widget)
{
  FILE* file = NULL;
  GtkWidget* dialog;
  GtkWidget* gl;

  if (!backend_mtp_server)
    {
      ggui_show_message ("warning", _("MTP Server not started"));
      return;
    }

  /* Create temp file */
  file = tmpfile();
  g_return_if_fail (file);

  /* Print MTP server info and rewind */
  mtp_server_print (backend_mtp_server, file);
  rewind (file);

  /* Create dialog */
  dialog = gnome_dialog_new (_("MTP Server Information"), GNOME_STOCK_BUTTON_OK, NULL);
  GTK_WINDOW(dialog)->position = GTK_WIN_POS_MOUSE;
  gnome_dialog_set_default (GNOME_DIALOG(dialog), 0);
  gtk_signal_connect (GTK_OBJECT(dialog), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), NULL);

  /* Create Gnome less */
  gl = gnome_less_new();
  gnome_less_show_filestream (GNOME_LESS(gl), file);
  gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(dialog)->vbox), gl, TRUE, TRUE, 0);
  gtk_widget_show (gl);

  gtk_widget_show (dialog);

  /* Close the file */
  if (file)
    fclose (file);
}

# endif /* JM_ENABLE_GNOME */



void
on_jm_homepage (gpointer callback_data, guint callback_action, GtkWidget* widget)
{
# ifdef JM_ENABLE_GNOME
  {
    gnome_url_show("http://www.junglemonkey.net");
  }
# endif
}


void
on_jm_manual (gpointer callback_data, guint callback_action, GtkWidget* widget)
{
# ifdef JM_ENABLE_GNOME
  {
    gchar *tmp;
    tmp = gnome_help_file_find_file ("jm", "jm.html");
    if (tmp) 
      {
	gnome_help_goto(0, tmp);
	g_free(tmp);
      }
  }
# endif
}


void
on_jm_faq (gpointer callback_data, guint callback_action, GtkWidget* widget)
{
# ifdef JM_ENABLE_GNOME
  {
    gchar *tmp;
    tmp = gnome_help_file_find_file ("jm", "faq.html");
    if (tmp) 
      {
	gnome_help_goto(0, tmp);
	g_free(tmp);
      }
  }
# endif
}


void
on_about(gpointer callback_data, guint callback_action, GtkWidget* widget)
{
  ggui_menu_about();
}





/* **************************************** */


void
gui_activate(JMFile* jmfile)
{
  g_return_if_fail (jmfile);

  /* Open URL's using the browser */
  if (jmfile->type == JMFILE_URL)
    {
      g_return_if_fail (jmfile->url);

      if (!strncmp(jmfile->url, "jmchat", 6))
	{
	  if (!ggui_chat_on(jmfile))
	    ggui_chat_join (jmfile);
	}
      else
	{
#         ifdef JM_ENABLE_GNOME
	  {
	    gnome_url_show (jmfile->url);
	  }
#         endif
	}

      return;
    }

# ifdef JM_ENABLE_GNOME
  {
    gchar* url;

    url = g_strconcat("file:", jmfile->path, NULL);
    gnome_url_show (url);
    g_free (url);
  }
# endif


#if 0

  /* I can't get this working right... */

  const gchar* mime_type;
  const gchar* cmd;
  gchar* argv[3];

  g_return_if_fail (jmfile != NULL);
  g_return_if_fail (jmfile->path != NULL);

  DEBUGP("gui_activate\n");

  mime_type = gnome_mime_type_of_file (jmfile->path);
  if (!mime_type)
    goto error;

  cmd = gnome_mime_get_value (mime_type, "open");
  if (!cmd)
    goto error;

  argv[0] = (gchar*) cmd;
  argv[1] = jmfile->path;
  argv[2] = NULL;

  if (gnome_execute_async (NULL, 3, argv) <= 0)
    goto error;

  return;

 error:

#endif
}


void
gui_add(JMFile* jmfile)
{
  g_return_if_fail (jmfile);

  DEBUGP("gui_add %s\n", jmfile->url);

  if (shutting_down)   return;

  ggui_channels_add (jmfile);
  ggui_chat_add (jmfile);
  ggui_download_add (jmfile);
  ggui_files_add (jmfile);
  ggui_properties_add (jmfile);
  ggui_search_add (jmfile);

}


void
gui_remove(JMFile* jmfile)
{
  g_return_if_fail (jmfile != NULL);

  DEBUGP("gui_remove %s\n", jmfile->url);

  if (shutting_down) return;

  ggui_channels_remove (jmfile);
  ggui_chat_remove (jmfile);
  ggui_download_remove (jmfile);
  ggui_files_remove (jmfile);
  ggui_properties_remove (jmfile);
  ggui_search_remove (jmfile);

  if (jmfile->gui_data)
    {
      g_free (jmfile->gui_data);
      jmfile->gui_data = NULL;
    }
}


void
gui_update(JMFile* jmfile)
{
  DEBUGP("gui_update\n");

  if (shutting_down) return;

  ggui_channels_update (jmfile);
  ggui_chat_update (jmfile);
  ggui_download_update (jmfile);
  ggui_files_update (jmfile);
  ggui_properties_update (jmfile);
  ggui_search_update (jmfile);

}


void
gui_freeze(void)
{
  DEBUGP("gui_freeze\n");

  if (shutting_down) return;

  ggui_channels_freeze ();
  ggui_chat_freeze ();
  ggui_download_freeze ();
  ggui_files_freeze ();
  ggui_properties_freeze ();
  ggui_search_freeze ();
}


void
gui_thaw(void)
{
  DEBUGP("gui_thaw\n");

  if (shutting_down) return;

  ggui_channels_thaw ();
  ggui_chat_thaw ();
  ggui_download_thaw ();
  ggui_files_thaw ();
  ggui_properties_thaw ();
  ggui_search_thaw ();
}
