/* Jungle Monkey
 * Copyright (C) 1999, 2000  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "ggui.h"
#include "ggui_chat.h"
#include "ggui_config.h"
#include "ggui_menu.h"
#include "jmchat.h"


/* ******************** */

static GladeXML* 	menu_xml = NULL;

static gchar*		nickname = NULL;

static GtkCList* 	chat_clist = NULL;
static GtkText* 	chat_text = NULL;
static GtkEntry* 	chat_entry = NULL;
static GtkContainer* 	chat_container = NULL;
static GtkWidget* 	chat_popup = NULL;

static JMFile*		selected_chat = NULL;

static GList*		bookmarked_chats = NULL;
static GList*		chats = NULL;	/* List of all chats, joined or not joined */


/* ******************** */

static void chat_add (JMFile* jmfile);

static void chat_info_cb (JMChat* chat, JMChatType type, gchar* nick, 
			  gchar* msg, gpointer user_data);
static void chat_error_cb (JMChat* chat, gpointer user_data);

static void my_text_insert (GtkText* text, gchar* str);


/* ************************************************************ */


void
ggui_chat_init(void)
{
  menu_xml = ggui_get_jm_glade_xml("chat_popup", "master_popup.glade");
  g_assert (menu_xml != NULL);
  glade_xml_signal_autoconnect(menu_xml);

  chat_clist = GTK_CLIST(glade_xml_get_widget(jmw_xml, "chat_clist"));
  chat_text  = GTK_TEXT(glade_xml_get_widget(jmw_xml, "chat_text"));
  chat_entry  = GTK_ENTRY(glade_xml_get_widget(jmw_xml, "chat_entry"));
  chat_container = GTK_CONTAINER(glade_xml_get_widget(jmw_xml, "chat_container"));

  chat_popup = GTK_WIDGET(glade_xml_get_widget(menu_xml, "chat_popup"));

  g_assert (chat_clist);
  g_assert (chat_text);
  g_assert (chat_entry);
  g_assert (chat_container);
  g_assert (chat_popup);

  gtk_widget_set_sensitive (GTK_WIDGET(chat_container), FALSE);

  nickname = ggui_chat_get_nickname ();

  /* TODO: tooltips */
/*    ggui_set_tooltip (GTK_WIDGET(channels_clist), _("Channels you have joined.")); */
}


void
ggui_chat_shutdown (void)
{
  char** argv;
  gint argc;
  GList* i;

  /* Delete GUI data of all known chats */
  for (i = chats; i != NULL; i = i->next)
    {
      JMFile* chat = (JMFile*) i->data;

      if (chat->gui_data)
	{
	  if (chat->gui_data->chat)
	    {
	      jmchat_leave (chat->gui_data->chat);
	      jmchat_delete (chat->gui_data->chat);
	      chat->gui_data->chat = NULL;
	    }

	  if (chat->gui_data->chat_text)
	    {
	      g_free (chat->gui_data->chat_text);
	      chat->gui_data->chat_text = NULL;
	    }
	}
    }
  g_list_free (chats);
  chats = NULL;


  /* Save bookmarks */
  argc = g_list_length (bookmarked_chats);
  argv = g_new0 (gchar*, argc + 1);

  for (argc = 0, i = bookmarked_chats; i != NULL; i = i->next)
    {
      JMFile* jmfile = (JMFile*) i->data;

      if (jmfile->url)
	argv[argc++] = jmfile->url;
    }

  ggui_config_set_vector ("/Jungle Monkey/General/Chat Bookmarks", argv);
  g_free (argv);

  /* Save nickname */
  if (nickname)
    ggui_config_set_string ("/Jungle Monkey/Chat/Nickname", nickname);

}


void
ggui_chat_load_bookmarks (void)
{
  gchar** argv;
  gint i;

  argv = ggui_config_get_vector ("/Jungle Monkey/General/Chat Bookmarks");
  if (!argv)
    return;

  for (i = 0; argv[i] != NULL; ++i)
    {
      JMFile* jmfile;

      /* Skip if NULL */
      if (!*argv[i])
	continue;

      /* Add it */
      jmfile = backend_url_add (argv[i], NULL);
      if (jmfile)
	{
	  chat_add (jmfile);
	  if (!g_list_find(bookmarked_chats, selected_chat))
	    bookmarked_chats = g_list_prepend (bookmarked_chats, jmfile);
	}
    }

  g_free (argv);
}




void
ggui_chat_add (JMFile* jmfile)
{
  g_return_if_fail (jmfile);

  /* Ignore non-jmchat's */
  if (jmfile->type != JMFILE_URL || strncmp(jmfile->url, "jmchat", 6))
    return;

  /* Ignore if we aren't on it */
  if (!jmfile->is_local && !jmfile_has_parent(jmfile, NULL))
    return;

  chat_add (jmfile);
}



static void
chat_add (JMFile* jmfile)
{
  gchar* texts[2] = {NULL, NULL};
  gint row;

  g_return_if_fail (jmfile);
  g_return_if_fail (jmfile->name);
  g_return_if_fail (jmfile->type == JMFILE_URL);
  g_return_if_fail (!strncmp(jmfile->url, "jmchat", 6));

  /* Ignore if we're already showing this chat */
  if (gtk_clist_find_row_from_data (chat_clist, jmfile) >= 0)
    return;

  texts[0] = jmfile->name;
  row = gtk_clist_append (chat_clist, texts);
  gtk_clist_set_row_data (chat_clist, row, jmfile);

  ggui_chat_update(jmfile);
}



void
ggui_chat_update (JMFile* jmfile)
{
  GdkPixmap* pixmap;
  GdkBitmap* mask;
  gint row;

  g_return_if_fail (jmfile);
  g_return_if_fail (jmfile->name);
  g_return_if_fail (jmfile->url);

  DEBUGP ("ggui_chat_update\n");

  /* Ignore non-chats */
  if (jmfile->type != JMFILE_URL || strncmp(jmfile->url, "jmchat", 6))
    return;
  
  /* Add if we haven't already */
  row = gtk_clist_find_row_from_data (chat_clist, jmfile);
  if (row < 0)
    {
      chat_add (jmfile);
      return;
    }

  ggui_get_pixmaps (jmfile, &pixmap, &mask);

  gtk_clist_set_pixtext (chat_clist, row, 0, 
			 jmfile->name, 5, pixmap, mask);

  /* Fix sensitivite if necessary */
  if (jmfile == selected_chat)
    gtk_widget_set_sensitive (GTK_WIDGET(chat_container), ggui_chat_on(jmfile));
}


void
ggui_chat_remove (JMFile* jmfile)
{
  gint row;

  g_return_if_fail (jmfile);

  /* Ignore non-chats */
  if (jmfile->type != JMFILE_URL || strncmp(jmfile->url, "jmchat", 6))
    return;

  bookmarked_chats = g_list_remove (bookmarked_chats, jmfile);
  chats = g_list_remove (chats, jmfile);

  /* Remove the chat from the list */
  row = gtk_clist_find_row_from_data (chat_clist, jmfile);
  if (row >= 0)
    gtk_clist_remove (chat_clist, row);

  /* Leave the chat if we're on it */
  if (jmfile->gui_data)
    {
      if (jmfile->gui_data->chat)
	{
	  jmchat_leave (jmfile->gui_data->chat);
	  jmchat_delete (jmfile->gui_data->chat);
	  jmfile->gui_data->chat = NULL;
	}

      if (jmfile->gui_data->chat_text)
	{
	  g_free (jmfile->gui_data->chat_text);
	  jmfile->gui_data->chat_text = NULL;
	}
    }
}


void
ggui_chat_freeze(void)
{
  gtk_clist_freeze (chat_clist);
}


void
ggui_chat_thaw(void)
{
  gtk_clist_thaw (chat_clist);
}



/* **************************************** */

gboolean
ggui_chat_on (JMFile* url)
{
  g_return_val_if_fail (url, FALSE);
  
  if (url->gui_data && 
      url->gui_data->chat && 
      jmchat_is_up (url->gui_data->chat))
    return TRUE;

  return FALSE;
}


gboolean
ggui_chat_is_down (JMFile* url)
{
  g_return_val_if_fail (url, FALSE);

  if (url->gui_data && 
      url->gui_data->chat &&
      url->gui_data->chat->is_down)
    return TRUE;

  return FALSE;
}


gboolean
ggui_chat_join (JMFile* url)
{
  g_return_val_if_fail (url, TRUE);
  g_return_val_if_fail (url->type == JMFILE_URL, TRUE);
  g_return_val_if_fail (!strncmp(url->url, "jmchat", 6), TRUE);

  if (ggui_chat_on (url))
    return FALSE;

  if (url->gui_data && url->gui_data->chat)
    {
      gboolean rv;

      rv = jmchat_rejoin (url->gui_data->chat);
      gui_update (url);
      return rv;
    }
  else
    {
      JMChat* chat;

      chat = jmchat_join (url->url, nickname);
      if (chat)
	{
	  GuiData* gui_data;

	  chat->info_func = chat_info_cb;
	  chat->error_func = chat_error_cb;
	  chat->user_data = url;

	  gui_data = ggui_get_gui_data (url);
	  gui_data->chat = chat;

	  chats = g_list_prepend (chats, url);

	  gui_update (url);
	}
      else
	return TRUE;
    }

  return FALSE;
}


void
ggui_chat_leave (JMFile* url)
{
  g_return_if_fail (url);
  
  if (!url->gui_data || !url->gui_data->chat)
    return;

  jmchat_leave (url->gui_data->chat);

  if (url->gui_data->chat_text)
    {
      g_free (url->gui_data->chat_text);
      url->gui_data->chat_text = NULL;
    }

  if (url == selected_chat)
    {
      gtk_editable_delete_text (GTK_EDITABLE(chat_text), 0, -1);
      gtk_widget_set_sensitive (GTK_WIDGET(chat_container), FALSE);
    }

  gui_update (url);
}


JMFile*
ggui_chat_create (gchar* name, JMFile* parent)
{
  JMFile* jmfile;
  JMChat* chat;
  gchar* fullname = NULL;

  g_return_val_if_fail (name, NULL);
  g_return_val_if_fail (!parent || 
			(parent->type == JMFILE_CHANNEL || 
			 parent->type == JMFILE_DIR), NULL);

  if (jmfile_get_child_by_name (parent, name))
    return NULL;

  fullname = jmfile_make_fullname (parent, name);

  chat = jmchat_create (fullname, nickname);
  if (chat)
    {
      jmfile = backend_url_create (name, chat->url, parent);
      if (jmfile)
	{
	  GuiData* gui_data;

	  gui_data = ggui_get_gui_data (jmfile);

	  gui_data->chat = chat;

	  chat->info_func = chat_info_cb;
	  chat->error_func = chat_error_cb;
	  chat->user_data = jmfile;

	  chats = g_list_prepend (chats, jmfile);

	  gui_update (jmfile);
	}
      else
	{
	  jmchat_delete (chat);
	}
    }

  g_free (fullname);

  return jmfile;
}



static void
chat_info_cb (JMChat* chat, JMChatType type, gchar* nick, 
	      gchar* msg, gpointer user_data)
{
  JMFile* url = (JMFile*) user_data;
  gchar* str = NULL;

  g_return_if_fail (url);
  g_return_if_fail (nick);

  DEBUGP ("chat_info_cb\n");

  switch (type)
    {
    case JMCHAT_JOIN:
      {
	time_t t;
	char* time_str;

	t = time(NULL);
	t = mktime(gmtime(&t));
	time_str = ctime(&t);		/* don't free string */
	time_str[strlen(time_str) - 1] = '\0';

	gdk_beep();

	str = g_strdup_printf (_("-> %s has joined at time %s\n"), nick, time_str);
	break;
      }

    case JMCHAT_LEAVE:
      {
	time_t t;
	char* time_str;

	t = time(NULL);
	t = mktime(gmtime(&t));
	time_str = ctime(&t);		/* don't free string */
	time_str[strlen(time_str) - 1] = '\0';

	str = g_strdup_printf (_("<- %s has left at time %s\n"), nick, time_str);
	break;
      }

    case JMCHAT_SAY:
      {
	g_return_if_fail (msg);
	str = g_strdup_printf (_("%s: %s\n"), nick, msg);
	break;
      }
      
    default: 
      break;;
    }

  if (str)
    {
      if (url == selected_chat)
	{
	  my_text_insert (chat_text, str);
	}
      else if (url->gui_data)
	{
	  if (url->gui_data->chat_text)
	    {
	      gchar* temp;

	      temp = g_strconcat (url->gui_data->chat_text, str, NULL);
	      g_free (url->gui_data->chat_text);
	      url->gui_data->chat_text = temp;
	    }
	  else
	    url->gui_data->chat_text = g_strdup (str);
	}

      g_free (str);
    }
}


static void
chat_error_cb (JMChat* chat, gpointer user_data)
{
  JMFile* url = (JMFile*) user_data;

  g_return_if_fail (url);

  gui_update (url);
}



gchar*
ggui_chat_get_nickname (void)
{
  /* Try using config */
  if (nickname == NULL)
    {
      nickname = ggui_config_get_string ("/Jungle Monkey/Chat/Nickname", NULL);

      /* Try g_get_user_name */
      if (!nickname && (nickname = g_get_user_name()))
	nickname = g_strdup (nickname);
      
      /* Try g_get_real_name */
      if (!nickname && (nickname = g_get_real_name()))
	nickname = g_strdup (nickname);

      if (!nickname)
	nickname = g_strdup ("Anonymous Monkey");
    }

  return nickname;
}



void
ggui_chat_set_nickname (gchar* nick)
{
  g_return_if_fail (nick);

  if (nickname)
    g_free (nickname);

  nickname = g_strdup (nick);
}



/* **************************************** */


void 
on_chat_select_row(GtkCList* list, gint row, gint column, GdkEvent* event)
{
  JMFile* new_selected_chat;

  new_selected_chat = (JMFile*) gtk_clist_get_row_data(list, row);
  if (new_selected_chat == selected_chat)
    return;

  /* Save the text of the old selected chat */
  if (selected_chat)
    {
      gchar* text;

      g_return_if_fail (selected_chat->gui_data);

      text = gtk_editable_get_chars (GTK_EDITABLE(chat_text), 0, -1);
      g_return_if_fail (text);
      
      if (selected_chat->gui_data->chat_text)
	g_free (selected_chat->gui_data->chat_text);

      selected_chat->gui_data->chat_text = text;
    }

  /* Restore the text of the new selected chat */
  if (new_selected_chat)
    {
      gtk_editable_delete_text (GTK_EDITABLE(chat_text), 0, -1);

      if (new_selected_chat->gui_data->chat_text)
	{
	  gtk_text_insert (chat_text, NULL, NULL, NULL, 
			   new_selected_chat->gui_data->chat_text,
			   strlen(new_selected_chat->gui_data->chat_text));
	  gtk_text_set_point (chat_text, gtk_text_get_length (chat_text));
	}
    }

  selected_chat = new_selected_chat;

  gtk_widget_set_sensitive (GTK_WIDGET(chat_container), 
			    (selected_chat != NULL && ggui_chat_on (selected_chat)));
}


void 
on_chat_unselect_row(GtkCList* list, gint row, gint column, GdkEvent* event)
{
  /* Save the text of the old selected chat */
  if (selected_chat)
    {
      gchar* text;

      g_return_if_fail (selected_chat->gui_data);

      text = gtk_editable_get_chars (GTK_EDITABLE(chat_text), 0, -1);
      g_return_if_fail (text);
      
      if (selected_chat->gui_data->chat_text)
	g_free (selected_chat->gui_data->chat_text);

      selected_chat->gui_data->chat_text = text;
    }


  selected_chat = NULL;
  gtk_widget_set_sensitive (GTK_WIDGET(chat_container), FALSE);
}



#define HIDE_MENUITEM(MI) do {       					\
  GtkWidget* menu_item = glade_xml_get_widget(menu_xml, (MI));		\
  g_return_val_if_fail (menu_item != NULL, TRUE);			\
  gtk_widget_hide(menu_item);} while (0)

#define SHOW_MENUITEM(MI) do {       					\
  GtkWidget* menu_item = glade_xml_get_widget(menu_xml, (MI));		\
  g_return_val_if_fail (menu_item != NULL, TRUE);			\
  gtk_widget_show(menu_item);} while (0)



/* Return TRUE if we handle event; FALSE to pass on */
gint
on_chat_button_press (GtkCList* list, GdkEventButton *event, gpointer user_data)
{
  gint row, column;
  gboolean rv = FALSE;

  /* User double clicked - activate the thing selected */
  if (event->type == GDK_2BUTTON_PRESS && 
      (event->button == 1 || event->button == 2))
    {
      /* Get the selection information */
      if (gtk_clist_get_selection_info(list, event->x, event->y, &row, &column))
	{
	  /* Select the clicked row, if there is one */
	  gtk_clist_select_row (list, row, 0);
	
	  /* Double clicks on chats activates them. */
	  if (selected_chat)
	    backend_activate (selected_chat);
	}
      else
	gtk_clist_unselect_all(list);

      rv = TRUE;
    }

  /* User right-clicked - show context menu */
  else if (event->type == GDK_BUTTON_PRESS && event->button == 3)
    {
      GList* i;

      /* Select the clicked row, if there is one */
      if (gtk_clist_get_selection_info (list, event->x, event->y, &row, &column))
	gtk_clist_select_row (list, row, 0);
      else
	gtk_clist_unselect_all(list);

      /* Hide everything */
      for (i = GTK_MENU_SHELL(chat_popup)->children; i != NULL; i = i->next)
	{
	  GtkWidget* menu_item = GTK_WIDGET(i->data);
	  g_return_val_if_fail (GTK_MENU_ITEM(menu_item) != NULL, FALSE);
	  gtk_widget_hide(menu_item);
	}

      SHOW_MENUITEM("open_mi");
      SHOW_MENUITEM("create_mi");

      /* Build the menu */
      if (selected_chat)
	{
	  SHOW_MENUITEM("separator1");

	  if (selected_chat->is_local)
	    SHOW_MENUITEM("remove_mi");
	  else if (ggui_chat_on (selected_chat))
	    {
	      SHOW_MENUITEM("leave_mi");
	      SHOW_MENUITEM("remove_mi");
	    }
	  else
	    {
	      SHOW_MENUITEM("join_mi");
	      SHOW_MENUITEM("remove_mi");
	    }

	  SHOW_MENUITEM("separator2");
	  SHOW_MENUITEM("properties_mi");

	  if (!selected_chat->is_local)
	    {
	      GtkCheckMenuItem* item = 
		GTK_CHECK_MENU_ITEM(glade_xml_get_widget(menu_xml, "bookmark_mi"));
	      g_return_val_if_fail (item != NULL, TRUE);

	      if (g_list_find(bookmarked_chats, selected_chat))
		gtk_check_menu_item_set_active (item, TRUE);
	      else
		gtk_check_menu_item_set_active (item, FALSE);

	      gtk_widget_show(GTK_WIDGET(item));
	    }
	}

      gtk_menu_popup (GTK_MENU(chat_popup), NULL, NULL, NULL, NULL, 3, event->time);

      rv = TRUE;
    }

  return rv;
}


void
on_chat_entry_activate (GtkEditable* editable, gpointer user_data)
{
  gchar* text;
  
  g_return_if_fail (chat_entry);
  g_return_if_fail (selected_chat);

  text = gtk_entry_get_text (chat_entry);

  if (text)
    {
      gchar* str;

      if (text[0] != '/')
	{
	  str = g_strdup_printf (_("%s: %s\n"), nickname, text);
	  my_text_insert (chat_text, str);
	  g_free (str);
	}
      else
	{
	  my_text_insert (chat_text, _("** / commands aren't supported yet.  Please email\n"
				       "** us if you're interested in contributing them.\n"));
	}

      if (ggui_chat_on(selected_chat))
	{
	  g_return_if_fail (selected_chat->gui_data && selected_chat->gui_data->chat);
	  
	  jmchat_say (selected_chat->gui_data->chat, text);
	}
    }

  gtk_entry_set_text (chat_entry, "");
}




void
on_chat_join (gpointer callback_data, guint callback_action, GtkWidget* widget)
{
  ggui_menu_join_chat (selected_chat);
}


void
on_chat_leave (gpointer callback_data, guint callback_action, GtkWidget* widget)
{
  ggui_menu_leave_chat (selected_chat);
}


void
on_chat_remove (gpointer callback_data, guint callback_action, GtkWidget* widget)
{
  g_return_if_fail (selected_chat);

  if (selected_chat->is_local)
    {
      ggui_menu_remove_chat (selected_chat);
    }
  else if (ggui_chat_on(selected_chat))
    {
      ggui_chat_leave (selected_chat);
      ggui_chat_remove (selected_chat);
    }
  else
    {
      ggui_chat_remove (selected_chat);
    }
}


void
on_chat_create (gpointer callback_data, guint callback_action, GtkWidget* widget)
{
  ggui_menu_create_private_chat ();
}


void
on_chat_open (gpointer callback_data, guint callback_action, GtkWidget* widget)
{
  ggui_menu_open_chat ();
}


void
on_chat_properties (gpointer callback_data, guint callback_action, GtkWidget* widget)
{
  ggui_menu_properties (selected_chat);
}


void
on_chat_bookmark(gpointer callback_data, guint callback_action, GtkWidget* widget)
{
  GtkCheckMenuItem* item = GTK_CHECK_MENU_ITEM(callback_data);

  g_return_if_fail (selected_chat != NULL);

  if (item->active)
    {
      if (!g_list_find(bookmarked_chats, selected_chat))
	bookmarked_chats = g_list_prepend(bookmarked_chats, selected_chat);
    }
  else
    {
      bookmarked_chats = g_list_remove(bookmarked_chats, selected_chat);
    }
}


/**

   What I want to do is this:
   
   If the scroll bar is at the bottom, insert the text and scroll down.

   If the scroll bar is not at the bottom, insert the text without
   moving the scroll bar.

   I can't figure out how to do this.

 */
static void
my_text_insert (GtkText* text, gchar* str)
{
/*    guint old_point; */
/*    guint old_position; */
/*    guint length; */
  guint str_len;

/*    gtk_text_freeze (text); */

/*    old_point = gtk_text_get_point (text); */
/*    old_position = gtk_editable_get_position (GTK_EDITABLE(text)); */
/*    length = gtk_text_get_length (text); */
  str_len = strlen(str);

/*    g_print ("OLD: position = %d, point = %d, length = %d, str_len = %d\n", */
/*  	   old_position, old_point, length, str_len); */
/*    g_print ("OLD vert adj value = %f, upper = %f, stepinc = %f, pageinc = %f, pagesize = %f\n", text->vadj->value, text->vadj->upper, text->vadj->step_increment, text->vadj->page_increment, text->vadj->page_size); */

  gtk_text_insert (text, NULL, NULL, NULL, str, str_len);
/*    gtk_editable_insert_text (GTK_EDITABLE(text), str, str_len, &length); */

/*    g_print ("NEW: position = %d, point = %d, length = %d\n", */
/*  	   gtk_editable_get_position (GTK_EDITABLE(text)),  */
/*  	   gtk_text_get_point (text),  */
/*  	   gtk_text_get_length (text)); */
/*    g_print ("NEW vert adj value = %f, upper = %f, stepinc = %f, pageinc = %f, pagesize = %f\n", text->vadj->value, text->vadj->upper, text->vadj->step_increment, text->vadj->page_increment, text->vadj->page_size); */

  /* Move point back if they had scrolled up */
/*    if (old_position == length) */
/*      gtk_editable_set_position (GTK_EDITABLE(text), length + str_len); */
  /* Otherwise move to the end */
/*    else */
/*      gtk_editable_set_position (GTK_EDITABLE(text), old_position); */

/*    if (old_point == length) */
/*      { */
/*        g_print ("set old point\n"); */
/*        gtk_text_set_point (text, length + str_len); */
/*      } */
/*    else */
/*      { */
/*        g_print ("set bottom point\n"); */
/*        gtk_text_set_point (text, old_point); */
/*      } */

/*    gtk_editable_changed (GTK_EDITABLE (text)); */

/*    gtk_text_thaw (text); */
}
