/* Jungle Monkey
 * Copyright (C) 1999, 2000  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "ggui.h"
#include "ggui_download.h"
#include "ggui_menu.h"
#include "util.h"


/* ******************** */

static GladeXML* 	menu_xml = NULL;

static GtkCList* 	download_clist = NULL;
static GtkWidget* 	download_popup = NULL;

static JMFile*		selected_file = NULL;


/* ******************** */

void on_download_select_row(GtkCList* list, gint row, gint column, GdkEvent* event);
void on_download_unselect_row(GtkCList* list, gint row, gint column, GdkEvent* event);
gint on_download_button_press (GtkCList* list, GdkEventButton *event, gpointer user_data);

void on_download_cancel_download(gpointer callback_data, guint callback_action, GtkWidget* widget);
void on_download_properties(gpointer callback_data, guint callback_action, GtkWidget* widget);




void
ggui_download_init(void)
{
  menu_xml = ggui_get_jm_glade_xml("download_popup", "master_popup.glade");
  g_assert (menu_xml != NULL);
  glade_xml_signal_autoconnect(menu_xml);

  download_clist = GTK_CLIST(glade_xml_get_widget(jmw_xml, "jmdownloadclist"));
  download_popup = GTK_WIDGET(glade_xml_get_widget(menu_xml, "download_popup"));

  g_assert (download_clist != NULL);
  g_assert (download_popup != NULL);

  gtk_clist_column_title_passive (download_clist, 0);
  gtk_clist_column_title_passive (download_clist, 1);

  ggui_set_tooltip (GTK_WIDGET(download_clist), _("Files you are downloading."));
}


void
ggui_download_add(JMFile* jmfile)
{
  ggui_download_update(jmfile);
}


void
ggui_download_update(JMFile* jmfile)
{
  GuiData* gui_data;

  g_return_if_fail (jmfile != NULL);

  if (jmfile->type != JMFILE_FILE)
    return;

  gui_data = ggui_get_gui_data(jmfile);

  /* If the node is now being downloaded, add it to the download list */
  if (jmfile->status == JMFILE_STATUS_TRANSFERRING)
    {
      gint row;
      gchar* fraction;
      gchar* str;
      GdkPixmap* pixmap;
      GdkBitmap* mask;

      /* Add to download list if we haven't already */
      if (!gui_data->in_download_list)
	{
	  gchar* texts[2] = {jmfile->name, NULL};

	  row = gtk_clist_insert (download_clist, 0, texts);
	  gtk_clist_set_row_data(download_clist, row, jmfile);

	  gui_data->in_download_list = TRUE;
	}
      
      row = gtk_clist_find_row_from_data(download_clist, jmfile);
      g_return_if_fail (row >= 0);

      fraction = filesize_fraction_to_string(jmfile_get_bytes_received(jmfile),
					     jmfile_get_length(jmfile));
      g_return_if_fail (fraction != NULL);

      str = g_strdup_printf("%d%% (%s)", 
			    jmfile_get_percent_complete(jmfile),
			    fraction);

      ggui_get_pixmaps(jmfile, &pixmap, &mask);

      ggui_clist_update_row2(download_clist, row,
			     jmfile->name, str, pixmap, mask);

      g_free (fraction);
      g_free (str);
    }

  /* Otherwise, if the file is not transfering (anymore?), but it is
     in the download list, then remove it from the download list. */
  else if (gui_data->in_download_list)
    {
      gint row;

      row = gtk_clist_find_row_from_data (download_clist, jmfile);
      if (row >= 0)
	gtk_clist_remove (download_clist, row);

      gui_data->in_download_list = FALSE;
    }
}


void
ggui_download_remove(JMFile* jmfile)
{
  GuiData* gui_data;

  g_return_if_fail (jmfile != NULL);

  gui_data = jmfile->gui_data;
  if (gui_data != NULL && gui_data->in_download_list)
    {
      gint row;

      row = gtk_clist_find_row_from_data(download_clist, jmfile);
      if (row >= 0)
	gtk_clist_remove(download_clist, row);

      gui_data->in_download_list = FALSE;
    }
}


void
ggui_download_freeze(void)
{
  gtk_clist_freeze (download_clist);
}


void
ggui_download_thaw(void)
{
  gtk_clist_thaw (download_clist);
}


/* **************************************** */


void 
on_download_select_row(GtkCList* list, gint row, gint column, GdkEvent* event)
{
  selected_file = (JMFile*) gtk_clist_get_row_data(list, row);
}


void 
on_download_unselect_row(GtkCList* list, gint row, gint column, GdkEvent* event)
{
  selected_file = NULL;
}



#define HIDE_MENUITEM(MI) do {       					\
  GtkWidget* menu_item = glade_xml_get_widget(menu_xml, (MI));		\
  g_return_val_if_fail (menu_item != NULL, TRUE);			\
  gtk_widget_hide(menu_item);} while (0)

#define SHOW_MENUITEM(MI) do {       					\
  GtkWidget* menu_item = glade_xml_get_widget(menu_xml, (MI));		\
  g_return_val_if_fail (menu_item != NULL, TRUE);			\
  gtk_widget_show(menu_item);} while (0)



/* Return TRUE if we handle event; FALSE to pass on */
gint
on_download_button_press (GtkCList* list, GdkEventButton *event, gpointer user_data)
{
  gint row, column;
  gboolean rv = FALSE;

  /* User double clicked - activate the thing selected */
  if (event->type == GDK_2BUTTON_PRESS && 
      (event->button == 1 || event->button == 2))
    {
      /* Get the selection information */
      if (gtk_clist_get_selection_info(list, event->x, event->y, &row, &column))
	{
	  /* Select the clicked row, if there is one */
	  gtk_clist_select_row (list, row, 0);
	
	  /* Double clicks on files and channels activate them.  We
	     don't do anything for directories because that might be
	     annoying. */
	  if (selected_file)
	    backend_activate(selected_file);
	}
      else
	gtk_clist_unselect_all(list);

      rv = TRUE;
    }

  /* User right-clicked - show context menu */
  else if (event->type == GDK_BUTTON_PRESS && event->button == 3)
    {
      GList* i;

      /* Select the clicked row, if there is one */
      if (gtk_clist_get_selection_info (list, event->x, event->y, &row, &column))
	gtk_clist_select_row (list, row, 0);
      else
	gtk_clist_unselect_all(list);

      /* Hide everything */
      for (i = GTK_MENU_SHELL(download_popup)->children; i != NULL; i = i->next)
	{
	  GtkWidget* menu_item = GTK_WIDGET(i->data);
	  g_return_val_if_fail (GTK_MENU_ITEM(menu_item) != NULL, FALSE);
	  gtk_widget_hide(menu_item);
	}

      /* Build the menu */
      if (selected_file)
	{
	  if (selected_file->status == JMFILE_STATUS_TRANSFERRING)
	    SHOW_MENUITEM("download_cancel_download_mi");

	  SHOW_MENUITEM("separator1");
	  SHOW_MENUITEM("properties_mi");

	  gtk_menu_popup (GTK_MENU(download_popup), NULL, NULL, NULL, NULL, 3, event->time);
	}

      rv = TRUE;
    }

  return rv;
}


void
on_download_cancel_download(gpointer callback_data, guint callback_action, GtkWidget* widget)
{
  ggui_menu_cancel_download(selected_file);
}


void
on_download_properties(gpointer callback_data, guint callback_action, GtkWidget* widget)
{
  ggui_menu_properties(selected_file);
}
