/* Jungle Monkey
 * Copyright (C) 1999, 2000  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "ggui.h"
#include "ggui_chat.h"
#include "ggui_menu.h"
#include "ggui_properties.h"
#include "util.h"



/* ************************************************************ */


void
ggui_menu_init (void)
{
#if 0		/* Never added to GTK version, so not needed */
  {
    GtkWidget* jm_homepage;
    GtkWidget* faq_online;;

    jm_homepage = glade_xml_get_widget(jmw_xml, "jm_homepage_mi");
    faq_online = glade_xml_get_widget(jmw_xml, "faq_online_mi");

    g_return_if_fail (jm_homepage != NULL);
    g_return_if_fail (faq_online != NULL);

    gtk_widget_hide (jm_homepage);
    gtk_widget_hide (faq_online);
  }
# endif
}


void
ggui_menu_enable_menubar(void)
{
  GtkWidget* open_channel;
  GtkWidget* create_root_channel;

  open_channel = glade_xml_get_widget(jmw_xml, "open_channel_mi");
  create_root_channel = glade_xml_get_widget(jmw_xml, "create_root_channel_mi");

  g_return_if_fail (open_channel != NULL);
  g_return_if_fail (create_root_channel != NULL);

  gtk_widget_show(open_channel);
  gtk_widget_show(create_root_channel);
}


void
ggui_menu_disable_menubar(void)
{
  GtkWidget* open_channel;
  GtkWidget* create_root_channel;;

  open_channel = glade_xml_get_widget(jmw_xml, "open_channel_mi");
  create_root_channel = glade_xml_get_widget(jmw_xml, "create_root_channel_mi");

  g_return_if_fail (open_channel != NULL);
  g_return_if_fail (create_root_channel != NULL);

  gtk_widget_hide(open_channel);
  gtk_widget_hide(create_root_channel);
}



/* ************************************************************ */
/* Entry code							*/

typedef void (*EntryDialogOkFunc)(gchar* text, gchar* text2);
typedef void (*EntryDialogCancelFunc)(void);

typedef struct _EntryDialog
{
  GtkWidget* dialog;
  GtkWidget* entry;
  GtkWidget* entry2;
  EntryDialogOkFunc ok_func;
  EntryDialogCancelFunc cancel_func;

} EntryDialog;

static EntryDialog* entry_dialog_new(gchar* title, gchar* entry_text, gchar* entry_text2,
				     EntryDialogOkFunc ok_func, EntryDialogCancelFunc cancel_func);

static void on_entry_dialog_ok(gpointer, EntryDialog* entry_dialog);
#ifdef JM_ENABLE_GNOME
static void on_entry_dialog_changed (GtkEditable* editable, EntryDialog* entry_dialog);
static void on_entry_dialog_clicked (GnomeDialog *dialog, gint button_number, EntryDialog* entry_dialog);
#else
static void on_entry_dialog_cancel(GtkButton* button, EntryDialog* entry_dialog);
#endif





EntryDialog*
entry_dialog_new(gchar* title, gchar* entry_text, gchar* entry_text2,
		 EntryDialogOkFunc ok_func, EntryDialogCancelFunc cancel_func)
{
  EntryDialog* entry_dialog;
  GtkWidget* dialog = NULL;
  GtkWidget* entry = NULL;
  GtkWidget* entry2 = NULL;
  GtkWidget* label = NULL;

  g_return_val_if_fail (title 		!= NULL, NULL);
  g_return_val_if_fail (entry_text 	!= NULL, NULL);
  g_return_val_if_fail (ok_func 	!= NULL, NULL);
  g_return_val_if_fail (cancel_func 	!= NULL, NULL);

  entry_dialog = g_new0(EntryDialog, 1);

#ifdef JM_ENABLE_GNOME
  {
    GtkWidget* vbox;
    GtkWidget* hbox;

    dialog = gnome_dialog_new (title, GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);
    GTK_WINDOW(dialog)->position = GTK_WIN_POS_MOUSE;
    gnome_dialog_set_default (GNOME_DIALOG(dialog), 1);

    vbox = gtk_vbox_new (FALSE, 2);
    gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(dialog)->vbox), vbox, TRUE, TRUE, 0);
    gtk_widget_show (vbox);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
    gtk_widget_show (hbox);

    label = gtk_label_new(entry_text);
    gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, FALSE, 4);
    gtk_widget_show(label);

    entry = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 4);
    gtk_widget_show(entry);

    if (entry_text2)
      {
	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
	gtk_widget_show (hbox);

	label = gtk_label_new (entry_text2);
	gtk_box_pack_start (GTK_BOX(hbox), label, TRUE, FALSE, 4);
	gtk_widget_show (label);

	entry2 = gtk_entry_new();
	gtk_box_pack_start (GTK_BOX(hbox), entry2, TRUE, TRUE, 4);
	gtk_widget_show (entry2);

	gtk_signal_connect (GTK_OBJECT(entry2), "changed",
			    GTK_SIGNAL_FUNC(on_entry_dialog_changed), (gpointer) entry_dialog);  
      }

    gtk_signal_connect (GTK_OBJECT(dialog), "clicked",
			GTK_SIGNAL_FUNC(on_entry_dialog_clicked), (gpointer) entry_dialog);  

    gtk_signal_connect (GTK_OBJECT(entry), "changed",
			GTK_SIGNAL_FUNC(on_entry_dialog_changed), (gpointer) entry_dialog);  

  }
#else
  {
    GtkWidget* vbox;
    GtkWidget* hbox;
    GtkWidget* button;

    dialog = gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(dialog), title);
    GTK_WINDOW(dialog)->position = GTK_WIN_POS_MOUSE;

    vbox = gtk_vbox_new (FALSE, 2);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), vbox, TRUE, TRUE, 0);
    gtk_widget_show (vbox);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
    gtk_widget_show (hbox);

    label = gtk_label_new(entry_text);
    gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, FALSE, 4);
    gtk_widget_show(label);

    entry = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 4);
    gtk_widget_show(entry);

    if (entry_text2)
      {
	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
	gtk_widget_show (hbox);

	label = gtk_label_new (entry_text2);
	gtk_box_pack_start (GTK_BOX(hbox), label, TRUE, FALSE, 4);
	gtk_widget_show (label);

	entry2 = gtk_entry_new();
	gtk_box_pack_start (GTK_BOX(hbox), entry2, TRUE, TRUE, 4);
	gtk_widget_show (entry2);
      }

    hbox = gtk_hbox_new(FALSE, 5);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
		       hbox, TRUE, TRUE, 0);
    gtk_widget_show(hbox);
  
    button = gtk_button_new_with_label (_("OK"));
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC(on_entry_dialog_ok), (gpointer) entry_dialog);  
    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 4);
    gtk_widget_show (button);

    button = gtk_button_new_with_label (_("Cancel"));
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC(on_entry_dialog_cancel), (gpointer) entry_dialog);  
    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 4);
    gtk_widget_show (button);
  }
#endif

  gtk_signal_connect (GTK_OBJECT (entry), "activate",
		      GTK_SIGNAL_FUNC(on_entry_dialog_ok), (gpointer) entry_dialog);  
  if (entry2)
    {
      gtk_signal_connect (GTK_OBJECT (entry2), "activate",
			  GTK_SIGNAL_FUNC(on_entry_dialog_ok), (gpointer) entry_dialog);
    }
  gtk_widget_show(dialog);

  entry_dialog->dialog = dialog;
  entry_dialog->entry  = entry;
  entry_dialog->entry2  = entry2;
  entry_dialog->ok_func = ok_func;
  entry_dialog->cancel_func = cancel_func;


  return entry_dialog;
}


static void
on_entry_dialog_ok (gpointer button_or_entry, EntryDialog* entry_dialog)
{
  gchar* text;
  gchar* text2 = NULL;

  text = gtk_entry_get_text(GTK_ENTRY(entry_dialog->entry));
  if (entry_dialog->entry2)
    text2 = gtk_entry_get_text(GTK_ENTRY(entry_dialog->entry2));

  if (text && text[0] != '\0')
    {
      gchar* new_text_dup;
      gchar* new_text2_dup = NULL;
      gchar* new_text;
      gchar* new_text2 = NULL;

      new_text_dup = g_strdup(text);
      new_text = g_strstrip(new_text_dup);
      if (text2)
	{
	  new_text2_dup = g_strdup(text2);
	  new_text2 = g_strstrip(new_text2_dup);
	}

      (*entry_dialog->ok_func)(new_text, new_text2);

      g_free (new_text_dup);
      g_free (new_text2_dup);
    }
  else
    (*entry_dialog->cancel_func)();

  gtk_widget_destroy(entry_dialog->dialog);
  g_free(entry_dialog);
}



#ifdef JM_ENABLE_GNOME

static void
on_entry_dialog_changed (GtkEditable* editable, EntryDialog* entry_dialog)
{
  gchar* text;

  text = gtk_entry_get_text(GTK_ENTRY(entry_dialog->entry));

  if (text && *text)
    gnome_dialog_set_default (GNOME_DIALOG(entry_dialog->dialog), 0);
  else
    gnome_dialog_set_default (GNOME_DIALOG(entry_dialog->dialog), 1);
}


static void
on_entry_dialog_clicked(GnomeDialog *dialog, gint button_number, EntryDialog* entry_dialog)
{
  if (button_number == 0)
    {
      gchar* text;
      gchar* text2 = NULL;

      text = gtk_entry_get_text(GTK_ENTRY(entry_dialog->entry));
      if (entry_dialog->entry2)
	text2 = gtk_entry_get_text(GTK_ENTRY(entry_dialog->entry2));

      if (text && text[0] != '\0')
	{
	  gchar* new_text_dup;
	  gchar* new_text2_dup = NULL;
	  gchar* new_text;
	  gchar* new_text2 = NULL;

	  new_text_dup = g_strdup(text);
	  new_text = g_strstrip(new_text_dup);
	  if (text2)
	    {
	      new_text2_dup = g_strdup(text2);
	      new_text2 = g_strstrip(new_text2_dup);
	    }

	  (*entry_dialog->ok_func)(new_text, new_text2);

	  g_free (new_text_dup);
	  g_free (new_text2_dup);
	}
      else
	(*entry_dialog->cancel_func)();
    }
  else
    (entry_dialog->cancel_func)();

  gtk_widget_destroy(entry_dialog->dialog);
  g_free(entry_dialog);
}


#else

static void
on_entry_dialog_cancel(GtkButton* button, EntryDialog* entry_dialog)
{
  (*entry_dialog->cancel_func)();

  gtk_widget_destroy(entry_dialog->dialog);
  g_free(entry_dialog);
}

#endif



/* ************************************************************ */
/* Create channel code						*/


static void create_channel_ok(gchar* channel_name, gchar* ignore);
static void create_channel_cancel(void);

static EntryDialog* create_channel_dialog = NULL;
static JMFile** create_channel_filep = NULL;


void
ggui_menu_create_channel(JMFile** _create_channel_filep)
{
  g_return_if_fail (_create_channel_filep != NULL);

  if (create_channel_dialog == NULL)
    {
      create_channel_filep = _create_channel_filep;

      create_channel_dialog = entry_dialog_new(_("Create channel"), 
					       _("Channel name"), NULL,
					       create_channel_ok,
					       create_channel_cancel);
    }
}


static void
create_channel_ok (gchar* channel_name, gchar* ignore)
{
  create_channel_dialog = NULL;

  g_return_if_fail (channel_name != NULL);
  g_return_if_fail (create_channel_filep != NULL);
  g_return_if_fail (*create_channel_filep != NULL);


  if (*create_channel_filep)
    {
      if (is_okfilename(channel_name))
	{
	  if (backend_channel_create (channel_name, *create_channel_filep, FALSE) == NULL)
	    ggui_show_message ("warning", _("Could not create channel \"%s\"\n"), channel_name);
	}
      else
	{
	  ggui_show_message ("warning", 
			     _("Channel name has weird letters in it (%s).  "
			       "I'm not going to add it.\n"), 
			     channel_name);
	}
    }

  create_channel_filep = NULL;
}


static void
create_channel_cancel(void)
{
  create_channel_dialog = NULL;
  create_channel_filep = NULL;
}



/* ************************************************************ */
/* Create root channel code					*/

static void create_root_channel_ok (gchar* channel_name, gchar* ignore);
static void create_root_channel_cancel(void);
static EntryDialog* create_root_channel_dialog = NULL;


void
ggui_menu_create_root_channel(void)
{
  if (create_root_channel_dialog == NULL)
    {
      create_root_channel_dialog = entry_dialog_new(_("Create private channel"), 
						    _("Private channel name"), NULL,
						    create_root_channel_ok,
						    create_root_channel_cancel);
    }
}


static void
create_root_channel_ok (gchar* channel_name, gchar* ignore)
{
  create_root_channel_dialog = NULL;

  g_return_if_fail (channel_name != NULL);

  if (is_okfilename(channel_name))
    {
      if (backend_channel_create (channel_name, NULL, FALSE) == NULL)
	ggui_show_message ("warning", _("Could not create private channel \"%s\"\n"), channel_name);
    }
  else
    {
      ggui_show_message ("warning",
			 _("Channel name has weird letters in it (%s).  "
			   "Try a different name."),
			 channel_name);
    }
}

static void
create_root_channel_cancel(void)
{
  create_root_channel_dialog = NULL;
}



/* ************************************************************ */
/* Create channel from directory code				*/

static void on_ccfd_ok(GtkButton* button, GtkFileSelection* fs);
static void on_ccfd_cancel(GtkButton* button, GtkFileSelection* fs);

static GtkFileSelection* ccfd_selector = NULL;
static JMFile** ccfd_filep = NULL;

static void ccfd_entry_ok (gchar* name, gchar* ignore);
static void ccfd_entry_cancel(void);

static EntryDialog* ccfd_entry = NULL;
static gchar* ccfd_path = NULL;


void
ggui_menu_create_channel_from_directory(JMFile** _ccfd_filep)
{
  if (ccfd_entry != NULL)
    {
      gtk_widget_show (GTK_WIDGET(ccfd_entry));
      return;
    }

  if (ccfd_selector == NULL)
    {
      ccfd_filep = _ccfd_filep;

      ccfd_selector = GTK_FILE_SELECTION(gtk_file_selection_new(_("Create channel from directory")));
      GTK_WINDOW(ccfd_selector)->position = GTK_WIN_POS_MOUSE;
      gtk_signal_connect (GTK_OBJECT (ccfd_selector), "destroy",
			  (GtkSignalFunc) gtk_widget_destroy, NULL);
      gtk_signal_connect (GTK_OBJECT (ccfd_selector->ok_button), "clicked", 
			  (GtkSignalFunc) on_ccfd_ok, NULL);
      gtk_signal_connect (GTK_OBJECT (ccfd_selector->cancel_button), "clicked",  
			  (GtkSignalFunc) on_ccfd_cancel, NULL);
    }

  gtk_widget_show (GTK_WIDGET(ccfd_selector));
}


static void
on_ccfd_ok (GtkButton* button, GtkFileSelection* fs) 
{
  if (ccfd_selector)
    {
      gchar* path = gtk_file_selection_get_filename(ccfd_selector);
      /* I think this is a reference... */

      if (path && path[0] && is_dir(path))
	{
	  gchar* basename;
	  guint len;

	  g_return_if_fail (ccfd_path == NULL);
	  g_return_if_fail (ccfd_entry == NULL);


	  ccfd_entry = entry_dialog_new(_("Name the channel"), 
					_("Channel name"), NULL,
					ccfd_entry_ok,
					ccfd_entry_cancel);

	  ccfd_path = g_strdup(path);

	  /* If the path ends with a /, remove it */
	  len = strlen(ccfd_path);
	  --len;
	  if (ccfd_path[len] == G_DIR_SEPARATOR)
	    ccfd_path[len] = 0;

	  /* Get the name of the directory */
	  basename = g_basename(ccfd_path);

	  /* Set the default name to the name of the directory */
	  gtk_entry_set_text(GTK_ENTRY(ccfd_entry->entry), basename);
	  /* Don't delete the basename - it's points into the
             directory name. */

	}

      gtk_widget_destroy (GTK_WIDGET(ccfd_selector));

      ccfd_selector = NULL;
    }
}


static void
on_ccfd_cancel (GtkButton* button, GtkFileSelection* fs)
{
  if (ccfd_selector)
    {
      gtk_widget_destroy(GTK_WIDGET(ccfd_selector));

      ccfd_filep = NULL;
      ccfd_selector = NULL;
    }
}


static void
ccfd_entry_ok (gchar* channel_name, gchar* ignore)
{
  g_return_if_fail (channel_name != NULL);
  g_return_if_fail (ccfd_filep != NULL);
  g_return_if_fail (ccfd_path != NULL);

  if (is_okfilename(channel_name))
    {
      JMFile* channel;

      if ((channel = backend_channel_create (channel_name, *ccfd_filep, FALSE)) == NULL)
	ggui_show_message ("warning", _("Could not create channel \"%s\"\n"), channel_name);
      else
	{
	  backend_files_create (ccfd_path, channel, TRUE);
	}
    }
  else
    {
      ggui_show_message ("warning", 
			 _("Channel name has weird letters in it (%s).  "
			   "I'm not going to add it.\n"), 
			 channel_name);
    }

  g_free (ccfd_path);	
  ccfd_path = NULL;
  ccfd_entry = NULL;
  ccfd_filep = NULL;
}


static void
ccfd_entry_cancel(void)
{
  g_free (ccfd_path);	
  ccfd_path = NULL;
  ccfd_entry = NULL;
  ccfd_filep = NULL;
}




/* ************************************************************ */
/* Open channel code						*/

static void open_channel_ok (gchar* url, gchar* ignore);
static void open_channel_cancel (void);
static EntryDialog* open_channel_dialog = NULL;


void
ggui_menu_open_channel(void)
{
  if (open_channel_dialog == NULL)
    {
      open_channel_dialog = entry_dialog_new(_("Open channel"), 
					     _("Channel URL"), NULL,
					     open_channel_ok,
					     open_channel_cancel);
    }
}


static void
open_channel_ok (gchar* url, gchar* ignore)
{
  JMFile* file;

  open_channel_dialog = NULL;

  file = backend_channel_add (url, NULL);
  if (file)
    {
      backend_activate (file);
    }

  else
    {
      ggui_show_message ("warning", 
			 _("Could not open %s "
			   "(did you specify a URL like jm://blah)\n"), 
			url);
    }
}


static void
open_channel_cancel(void)
{
  open_channel_dialog = NULL;
}



/* ************************************************************ */
/* Create directory code						*/

static void create_directory_ok (gchar* name, gchar* ignore);
static void create_directory_cancel(void);

static EntryDialog* create_directory_dialog = NULL;
static JMFile** create_directory_filep = NULL;


void
ggui_menu_create_directory(JMFile** _create_directory_filep)
{
  if (create_directory_dialog == NULL)
    {
      create_directory_filep = _create_directory_filep;

      create_directory_dialog = entry_dialog_new(_("Create directory"), 
						 _("Directory name"), NULL,
						 create_directory_ok,
						 create_directory_cancel);
    }
}


static void
create_directory_ok (gchar* directory_name, gchar* ignore)
{
  g_return_if_fail (create_directory_filep != NULL);

  if (directory_name && *create_directory_filep)
    {
      if (is_okfilename(directory_name))
	{
	  if (backend_directory_create (directory_name, *create_directory_filep) == NULL)
	    ggui_show_message ("warning", _("Could not create directory \"%s\"\n"), directory_name);
	}
      else
	{
	  ggui_show_message ("warning", 
			     _("Directory name has weird letters in it (%s).  "
			       "Try a different name.\n"), 
			     directory_name);
	}
    }

  create_directory_dialog = NULL;
  create_directory_filep = NULL;
}


static void
create_directory_cancel(void)
{
  create_directory_dialog = NULL;
  create_directory_filep = NULL;
}



/* ************************************************************ */
/* Add file code						*/

static void on_add_file_ok(GtkButton* button, GtkFileSelection* fs);
static void on_add_file_cancel(GtkButton* button, GtkFileSelection* fs);

static GtkFileSelection* file_selector = NULL;
static JMFile** add_file_filep = NULL;

void
ggui_menu_add_file(JMFile** _add_file_filep)
{
  if (file_selector == NULL)
    {
      add_file_filep = _add_file_filep;

      file_selector = GTK_FILE_SELECTION(gtk_file_selection_new(_("Add file to channel")));
      GTK_WINDOW(file_selector)->position = GTK_WIN_POS_MOUSE;
      gtk_signal_connect (GTK_OBJECT (file_selector), "destroy",
			  (GtkSignalFunc) gtk_widget_destroy, NULL);
      gtk_signal_connect (GTK_OBJECT (file_selector->ok_button), "clicked", 
			  (GtkSignalFunc) on_add_file_ok, NULL);
      gtk_signal_connect (GTK_OBJECT (file_selector->cancel_button), "clicked",  
			  (GtkSignalFunc) on_add_file_cancel, NULL);
    }

  gtk_widget_show (GTK_WIDGET(file_selector));
}


static void
on_add_file_ok (GtkButton* button, GtkFileSelection* fs) 
{
  if (file_selector)
    {
      if (*add_file_filep)
	{
	  gchar* path = gtk_file_selection_get_filename(file_selector);

	  if (path && path[0] && !is_dir(path))
	    backend_files_create (path, *add_file_filep, FALSE);
	}

      gtk_widget_destroy (GTK_WIDGET(file_selector));

      add_file_filep = NULL;
      file_selector = NULL;
    }
}


static void
on_add_file_cancel (GtkButton* button, GtkFileSelection* fs)
{
  if (file_selector)
    {
      gtk_widget_destroy(GTK_WIDGET(file_selector));

      add_file_filep = NULL;
      file_selector = NULL;
    }
}


/* ************************************************************ */
/* Add directory code						*/

static void on_add_directory_ok(GtkButton* button, GtkFileSelection* fs);
static void on_add_directory_cancel(GtkButton* button, GtkFileSelection* fs);

static GtkFileSelection* directory_selector = NULL;
static JMFile** add_directory_filep = NULL;

void
ggui_menu_add_directory(JMFile** _add_directory_filep)
{
  if (directory_selector == NULL)
    {
      add_directory_filep = _add_directory_filep;

      directory_selector = GTK_FILE_SELECTION(gtk_file_selection_new(_("Add directory to channel")));
      GTK_WINDOW(directory_selector)->position = GTK_WIN_POS_MOUSE;
      gtk_signal_connect (GTK_OBJECT (directory_selector), "destroy",
			  (GtkSignalFunc) gtk_widget_destroy, NULL);
      gtk_signal_connect (GTK_OBJECT (directory_selector->ok_button), "clicked", 
			  (GtkSignalFunc) on_add_directory_ok, NULL);
      gtk_signal_connect (GTK_OBJECT (directory_selector->cancel_button), "clicked",  
			  (GtkSignalFunc) on_add_directory_cancel, NULL);
    }

  gtk_widget_show (GTK_WIDGET(directory_selector));
}


static void
on_add_directory_ok (GtkButton* button, GtkFileSelection* fs) 
{
  if (directory_selector)
    {
      if (*add_directory_filep)
	{
	  gchar* path = gtk_file_selection_get_filename(directory_selector);

	  if (path && path[0] && is_dir(path))
	    backend_files_create (path, *add_directory_filep, FALSE);
	}

      gtk_widget_destroy (GTK_WIDGET(directory_selector));

      add_directory_filep = NULL;
      directory_selector = NULL;
    }
}


static void
on_add_directory_cancel (GtkButton* button, GtkFileSelection* fs)
{
  if (directory_selector)
    {
      gtk_widget_destroy(GTK_WIDGET(directory_selector));

      add_directory_filep = NULL;
      directory_selector = NULL;
    }
}




/* ************************************************************ */
/* Add URL code							*/


static void add_url_ok (gchar* url, gchar* ignore);
static void add_url_cancel(void);

static EntryDialog* add_url_dialog = NULL;
static JMFile** add_url_filep = NULL;


void
ggui_menu_add_url (JMFile** _add_url_filep)
{
  if (add_url_dialog == NULL)
    {
      add_url_filep = _add_url_filep;

      add_url_dialog = entry_dialog_new(_("Add URL"), _("URL"), _("Name"),
					add_url_ok, add_url_cancel);
    }

}


static void
add_url_ok (gchar* url, gchar* name)
{
  g_return_if_fail (add_url_filep);

  if (url && name && *add_url_filep)
    {
      if (!backend_url_create (name, url, *add_url_filep))
	ggui_show_message ("warning", _("Could not create URL \"%s\"\n"), url);
    }

  add_url_dialog = NULL;
  add_url_filep = NULL;
}


static void
add_url_cancel(void)
{
  add_url_dialog = NULL;
  add_url_filep = NULL;
}


/* ************************************************************ */
/* Open chat code						*/

static void open_chat_ok (gchar* url, gchar* ignore);
static void open_chat_cancel (void);
static EntryDialog* open_chat_dialog = NULL;


void
ggui_menu_open_chat(void)
{
  if (open_chat_dialog == NULL)
    {
      open_chat_dialog = entry_dialog_new(_("Open chat"), 
					     _("Chat URL"), NULL,
					     open_chat_ok,
					     open_chat_cancel);
    }
}


static void
open_chat_ok (gchar* url, gchar* ignore)
{
  JMFile* file;

  g_return_if_fail (url);

  open_chat_dialog = NULL;

  if (!strncmp(url, "jmchat", 6) && (file = backend_url_add (url, NULL)))
    {
      backend_activate (file);
    }

  else
    {
      ggui_show_message ("warning", 
			 _("Could not open %s "
			   "(did you specify a URL like jmchat://blah)\n"), 
			url);
    }
}


static void
open_chat_cancel(void)
{
  open_chat_dialog = NULL;
}


/* ************************************************************ */
/* Create Chat							*/


static void create_chat_ok (gchar* chat_name, gchar* ignore);
static void create_chat_cancel (void);

static EntryDialog* create_chat_dialog = NULL;
static JMFile** create_chat_filep = NULL;


void
ggui_menu_create_chat (JMFile** _create_chat_filep)
{
  g_return_if_fail (_create_chat_filep);

  if (create_chat_dialog == NULL)
    {
      create_chat_filep = _create_chat_filep;

      create_chat_dialog = entry_dialog_new(_("Create chat"), 
					       _("Chat name"), NULL,
					       create_chat_ok,
					       create_chat_cancel);
    }
}


static void
create_chat_ok (gchar* chat_name, gchar* ignore)
{
  g_return_if_fail (chat_name);
  g_return_if_fail (create_chat_filep);
  g_return_if_fail (*create_chat_filep);


  if (*create_chat_filep)
    {
      if (ggui_chat_create (chat_name, *create_chat_filep) == NULL)
	{
	    ggui_show_message ("warning", _("Could not create chat \"%s\"\n"), chat_name);
	}
    }

  create_chat_dialog = NULL;
  create_chat_filep = NULL;
}


static void
create_chat_cancel(void)
{
  create_chat_dialog = NULL;
  create_chat_filep = NULL;
}



/* ************************************************************ */
/* Create Private Chat						*/

static void create_private_chat_ok (gchar* chat_name, gchar* ignore);
static void create_private_chat_cancel(void);
static EntryDialog* create_private_chat_dialog = NULL;


void
ggui_menu_create_private_chat (void)
{
  if (create_private_chat_dialog == NULL)
    {
      create_private_chat_dialog = entry_dialog_new(_("Create private chat"), 
						    _("Private chat name"), NULL,
						    create_private_chat_ok,
						    create_private_chat_cancel);
    }
}


static void
create_private_chat_ok (gchar* chat_name, gchar* ignore)
{
  create_private_chat_dialog = NULL;

  g_return_if_fail (chat_name != NULL);

  if (ggui_chat_create (chat_name, NULL) == NULL)
    ggui_show_message ("warning", _("Could not create private chat \"%s\"\n"), chat_name);
}


static void
create_private_chat_cancel(void)
{
  create_private_chat_dialog = NULL;
}



/* ************************************************************ */
/* Other GUI menu code						*/


void
ggui_menu_open_url (JMFile* jmfile)
{
  g_return_if_fail (jmfile);
  g_return_if_fail (jmfile->type == JMFILE_URL);

  backend_activate (jmfile);
}

void
ggui_menu_remove_url (JMFile* jmfile)
{
  g_return_if_fail (jmfile);
  g_return_if_fail (jmfile->type == JMFILE_URL);
  g_return_if_fail (jmfile->is_local);

  backend_url_remove (jmfile);
}




void
ggui_menu_get_file(JMFile* jmfile)
{
  g_return_if_fail (jmfile != NULL);
  g_return_if_fail (jmfile->type == JMFILE_FILE || jmfile->type == JMFILE_DIR);

  backend_activate(jmfile);
}


void
ggui_menu_cancel_download(JMFile* file)
{
  g_return_if_fail (file != NULL);
  g_return_if_fail (file->type == JMFILE_FILE);

  if (file->status == JMFILE_STATUS_TRANSFERRING)
    {
      backend_file_cancel (file);
    }
}


void 
ggui_menu_remove_file(JMFile* file)
{
  g_return_if_fail (file != NULL);
  g_return_if_fail (file->is_local);

  backend_remove (file);
}


void
ggui_menu_join_channel(JMFile* channel)
{
  g_return_if_fail (channel != NULL);
  g_return_if_fail (channel->type == JMFILE_CHANNEL);

  if (channel->btsmp == NULL)
    {
      backend_channel_join (channel);
    }
}


void
ggui_menu_remove_channel(JMFile* channel)
{
  g_return_if_fail (channel != NULL);
  g_return_if_fail (channel->type == JMFILE_CHANNEL);
  g_return_if_fail (channel->is_local);

  backend_channel_remove (channel);
}


void
ggui_menu_leave_channel(JMFile* jmfile)
{
  g_return_if_fail (jmfile != NULL);
  g_return_if_fail (jmfile->type == JMFILE_CHANNEL);
  g_return_if_fail (!jmfile->is_local);

  backend_channel_leave (jmfile);
}


void
ggui_menu_join_chat (JMFile* jmfile)
{
  g_return_if_fail (jmfile);
  g_return_if_fail (jmfile->type == JMFILE_URL);
  g_return_if_fail (jmfile->url);
  g_return_if_fail (!strncmp(jmfile->url, "jmchat", 6));

  ggui_chat_join (jmfile);
}


void
ggui_menu_remove_chat (JMFile* jmfile)
{
  g_return_if_fail (jmfile);
  g_return_if_fail (jmfile->type == JMFILE_URL);
  g_return_if_fail (jmfile->url);
  g_return_if_fail (!strncmp(jmfile->url, "jmchat", 6));
  g_return_if_fail (jmfile->is_local);

  backend_url_remove (jmfile);
}


void
ggui_menu_leave_chat (JMFile* jmfile)
{
  g_return_if_fail (jmfile);
  g_return_if_fail (jmfile->type == JMFILE_URL);
  g_return_if_fail (jmfile->url);
  g_return_if_fail (!strncmp(jmfile->url, "jmchat", 6));

  ggui_chat_leave (jmfile);
}



void
ggui_menu_properties(JMFile* jmfile)
{
  g_return_if_fail (jmfile != NULL);
  ggui_properties_show(jmfile);
}


void
ggui_menu_about(void)
{
#ifdef JM_ENABLE_GNOME
  GtkWidget* about;
  const gchar* authors[] = {_("David A. Helder"), NULL};

  about = gnome_about_new (_("Jungle Monkey"),
			   VERSION,
			   _("Copyright (C) 2000 The Regents of the University of Michigan"),
			   authors,
			   NULL,		/* Comment */
			   NULL 		/* Logo	*/
			   );
  gnome_dialog_set_parent (GNOME_DIALOG(about), GTK_WINDOW(jmw));
  gtk_widget_show(about);
#endif
}

