/* Jungle Monkey
 * Copyright (C) 1999, 2000  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "ggui.h"
#include "ggui_properties.h"


static GList* properties_shown = NULL;


gboolean on_properties_dialog_destroy (GtkWidget* widget, gpointer user_data);
void	 on_properties_button_clicked (GtkButton* button);




void
ggui_properties_init(void)
{
}


void
ggui_properties_shutdown (void)
{
  GList* i;

  for (i = properties_shown; i != NULL; )
    {
      JMFile* jmfile = (JMFile*) i->data;
      i = i->next;

      ggui_properties_remove (jmfile);
    }

  g_list_free (properties_shown);
  properties_shown = NULL;
}


void
ggui_properties_show (JMFile* jmfile)
{
  GuiData* gui_data;

  g_return_if_fail (jmfile);
  g_return_if_fail (jmfile->name);

  gui_data = ggui_get_gui_data(jmfile);
  if (!gui_data->properties_dialog)
    {
      gchar* widget_name;

      GladeXML* properties_xml;
      GtkWidget* properties_dialog;
      GtkLabel* properties_label;
      GtkText* properties_text;
      GtkObject* properties_button;

      gchar* title;

#     ifdef JM_ENABLE_GNOME
        widget_name = "properties_dialog_gnome";
#     else
        widget_name = "properties_dialog";
#     endif

      properties_xml = ggui_get_jm_glade_xml(widget_name, "properties_dialog.glade");
      g_return_if_fail (properties_xml != NULL);
      glade_xml_signal_autoconnect(properties_xml);

      properties_dialog = GTK_WIDGET(glade_xml_get_widget(properties_xml, widget_name));
      properties_label  = GTK_LABEL( glade_xml_get_widget(properties_xml, "properties_label"));
      properties_text   = GTK_TEXT(  glade_xml_get_widget(properties_xml, "properties_text"));
      properties_button = GTK_OBJECT(glade_xml_get_widget(properties_xml, "properties_button"));

      if (!properties_dialog)
	{
	  ggui_show_message ("error", _("Could not create properties dialog.\n"
					"Either JM is not correctly installed or libglade is\n"
					"out-of-date."));
	  return;
	}

      g_return_if_fail (properties_dialog != NULL);
      g_return_if_fail (properties_label  != NULL);
      g_return_if_fail (properties_text   != NULL);
      g_return_if_fail (properties_button   != NULL);

      gui_data->properties_dialog = properties_dialog;
      gui_data->properties_label  = properties_label;
      gui_data->properties_text   = properties_text;

      gtk_object_set_user_data (GTK_OBJECT(properties_dialog), jmfile);
      gtk_object_set_user_data (properties_button, jmfile);
 
      title = g_strdup_printf (_("%s Properties"), jmfile->name);
      gtk_window_set_title(GTK_WINDOW(properties_dialog), title);
      g_free (title);

      GTK_WINDOW(properties_dialog)->position = GTK_WIN_POS_MOUSE;
   }

  ggui_properties_update (jmfile);
  gtk_widget_show (gui_data->properties_dialog);

  if (!g_list_find (properties_shown, jmfile))
    properties_shown = g_list_prepend (properties_shown, jmfile);
}


void
ggui_properties_add(JMFile* jmfile)
{
  ggui_properties_update(jmfile);
}


void
ggui_properties_update(JMFile* jmfile)
{
  GuiData* gui_data;
  guint length;
  gchar* str = NULL;

  g_assert (jmfile != NULL);

  gui_data = ggui_get_gui_data(jmfile);
  if (gui_data->properties_dialog == NULL)
    return;

  /* Set the label */
  gtk_label_set_text (gui_data->properties_label, jmfile->name);

  /* Delete everything in the text box */
  gtk_text_set_point (gui_data->properties_text, 0);
  length = gtk_text_get_length (gui_data->properties_text);
  gtk_text_forward_delete (gui_data->properties_text, length);

  /* Write the stuff in the text box */
  switch (jmfile->type)
    {
    case JMFILE_CHANNEL:
      {
	gchar* temp;

	if (jmfile->is_local)
	  str = g_strdup_printf(_("My Channel %s\n"), jmfile->name);
	else
	  str = g_strdup_printf(_("Channel %s\n"), jmfile->name);

	temp = str;
	if (jmfile->is_channels_only)
	  str = g_strdup_printf(_("%sPolicy: Channel advertisements only\n"), str);
	else
	  str = g_strdup_printf(_("%sPolicy: All advertisements allowed\n"), str);
	g_free(temp);

	break;
      }
		
    case JMFILE_DIR:

      if (jmfile->is_local)
	str = g_strdup_printf(_("My Directory %s\n"), jmfile->name);
      else
	str = g_strdup_printf(_("Directory %s\n"), jmfile->name);

      break;

    case JMFILE_FILE:
      {
	if (jmfile->mtp_mirror)
	  {
	    str = g_strdup_printf(_("My File %s\n" 
				    "Length: %d\n"
				    "Number of downloads: %d\n"),
				  jmfile->name, jmfile->length,
				  jmfile->mtp_mirror->num_downloads);
	  }
	else
	  {
	    str = g_strdup_printf(_("File %s\n" 
				    "Length: %d\n"), 
				  jmfile->name, jmfile->length);
	  }

	if (jmfile->sha)
	  {
	    guchar sha[2 * GNET_SHA_HASH_LENGTH + 1];
	    gchar* temp;

	    gnet_sha_copy_string (jmfile->sha, sha);
	    sha[2 * GNET_SHA_HASH_LENGTH] = '\0';

	    temp = str;
	    str = g_strdup_printf(_("%sSHA: %s\n"), str, sha);
	    g_free (temp);
	  }

	break;
      }

    case JMFILE_URL:
      {
	if (jmfile->is_local)
	  {
	    str = g_strdup_printf(_("My URL %s\n"
				    "%s\n"),
				  jmfile->name, jmfile->url);
	  }
	else
	  {
	    str = g_strdup_printf(_("URL %s\n"
				    "%s\n"),
				  jmfile->name, jmfile->url);
	  }
	break;
      }

    default:
      g_return_if_fail(FALSE);
    }

  if (jmfile->url && jmfile->type != JMFILE_URL)
    {
      gchar* temp = str;
      str = g_strdup_printf(_("%sURL: %s\n"), str, jmfile->url);
      g_free (temp);
    }

  gtk_text_insert (gui_data->properties_text, NULL, NULL, NULL, str, strlen(str));
  g_free(str);
}


void
ggui_properties_remove(JMFile* jmfile)
{
  GuiData* gui_data;

  g_assert (jmfile);

  gui_data = jmfile->gui_data;
  if (gui_data && gui_data->properties_dialog)
    {
      gtk_widget_destroy (gui_data->properties_dialog);

      gui_data->properties_dialog = NULL;
      gui_data->properties_label  = NULL;
      gui_data->properties_text   = NULL;

      properties_shown = g_list_remove (properties_shown, jmfile);
    }
}


void
ggui_properties_freeze(void)
{
  return;
}


void 
ggui_properties_thaw(void)
{
  return;
}


/* ************************************************************ */


gboolean
on_properties_dialog_destroy (GtkWidget* widget, gpointer user_data)
{
  JMFile* jmfile;

  jmfile = (JMFile*) gtk_object_get_user_data(GTK_OBJECT(widget));
  g_return_val_if_fail (jmfile != NULL, FALSE);

  ggui_properties_remove (jmfile);

  return TRUE;
}


void
on_properties_button_clicked (GtkButton* button)
{
  JMFile* jmfile;

  jmfile = (JMFile*) gtk_object_get_user_data(GTK_OBJECT(button));
  g_return_if_fail (jmfile != NULL);

  ggui_properties_remove (jmfile);
}
