/* Jungle Monkey
 * Copyright (C) 1999, 2000  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _GUI_H
#define _GUI_H


#include "backend.h"



/**

  GUI interface.  Anything that uses the Jungle Monkey backend must
  implement these calls and structs.  (Though write us before you
  start on a GUI please.)

  upcalls:
    gui_add(File*)
    gui_remove(File*)
    gui_update(File*)

    gui_freeze()
    gui_thaw()

  struct:
    _GuiData

 **/



/**

   Activate the file.  This is called by backend_activate if the
   backend thinks the GUI should handle activation.

 */
void gui_activate (JMFile* jmfile);


/**

   Add this file to the GUI.  If the GUI already has the file, it
   should update the file.

 */
void gui_add (JMFile* jmfile);


/**

   Remove the file from the GUI.  If the GUI doesn't have the file, it
   should do nothing.

 */
void gui_remove (JMFile* jmfile);


/**

   Update the specified file.  If the GUI doesn't have the file, it
   should do nothing.

 */
void gui_update (JMFile* jmfile);


#if 0	/* not sure if the backend will use this or not... */
/**

   Show a message to the user.  The message types are: info, warning,
   error, question, and generic.

 */
void gui_show_message (const gchar* type, const gchar* format, ...);
#endif


/**

   Freeze/thaw the GUI.  This pair of calls is used when their is a
   series of GUI calls.  gui_freeze is a hint that the GUI should not
   draw anything to the screen, gui_thaw is a hint that it can draw
   again.

 */
void gui_freeze (void);
void gui_thaw (void);



/**

   Each JMFile has a pointer to a _GuiData struct.  The GUI can use
   for anything it wants - it is never touched by the backend.

 */
struct _GuiData;



#endif /* _GUI_H */
