/* Jungle Monkey
 * Copyright (C) 1999, 2000  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**

   JMP is the Jungle Monkey Protocol.

   It looks like this:
     name attribute2="value1" attribute2 attribute3=4
     name2 
     name3 attribute4

   We use it like this in Jungle Monkey:
     ADD name="/channel/myfile" url="jmftp://blah"
     UPDATE name="/channel/otherfile" length="23"

   And we also use it for the search server:
     SEARCH string="flav"			[sent to server]
     MATCH name="flavor.jpg" url="blah"		[received from server]

   So, it's pretty handy.  We might be able to hack it up to work with
   other line-based protocols like NNTP and stick it in GNet.

*/


#ifndef _JMP_H
#define _JMP_H

#include <glib.h>


typedef struct _JMPAnnouncement
{
  gchar* name;
  GList* attributes;
  /* TODO: Children? */

} JMPAnnouncement;


typedef struct _JMPAttribute
{
  gchar* name;
  gchar* value;

} JMPAttribute;


/* 

   NOTES

   Do not delete the string returned by jmp_announcement_get_attribute.  
   jmp_announcement_delete deletes the list too.

*/


JMPAnnouncement* jmp_announcement_new(gchar* name);
void		 jmp_announcement_delete(JMPAnnouncement* announcement);
void		 jmp_announcement_delete_list(GList* list);

void		 jmp_announcement_set_attribute(JMPAnnouncement* announcement, gchar* name, gchar* value);
gchar*	 	 jmp_announcement_get_attribute(JMPAnnouncement* announcement, gchar* name);
gboolean	 jmp_announcement_has_attribute(JMPAnnouncement* announcement, gchar* name);
void		 jmp_announcement_remove_attribute(JMPAnnouncement* announcement, gchar* name);

GList*		 jmp_read(gchar* buffer, gint length);
void		 jmp_write_list(GList* list, gchar** bufferp, gint* lengthp);
void		 jmp_write(JMPAnnouncement* announcement, gchar** bufferp, gint* lengthp);

/*  guint		 jmp_size(JMPAnnouncement* announcement); */
/*  guint		 jmp_size_list(GList* list); */


#endif /* _JMP_H */
